@@ -184,8 +184,15 @@ extern bool newloc_r; /* location replaces this page */
 extern const char *ebrc_string; /* default ebrc file */
 
 struct eb_curl_callback_data {
+// where to put the captured data.
 	char **buffer;
 	int *length;
+// State of download to disk, see http.c for state values.
+	int down_state;
+	int down_fd;	/* downloading file descriptor */
+	const char *down_file;	/* downloading filename */
+	const char *down_file2;	/* without download directory */
+	int down_length;
 };
 
 struct MACCOUNT {		/* pop3 account */
@@ -225,9 +232,12 @@ struct DBTABLE {
 	char *nullable;
 };
 
-/* various globals */
-extern CURL *http_curl_handle;
+// This curl handle is always open, to retain the cookie space
+// and to accept cookies generated by javascript.
+extern CURL *global_http_handle;
+extern CURLSH *global_share_handle;
 extern int debugLevel;		/* 0 to 9 */
+extern FILE *debugFile;
 extern char *sslCerts;		/* ssl certificates to validate the secure server */
 extern int verifyCertificates;	/* is a certificate required for the ssl connection? */
 extern int displayLength;	/* when printing a line */
@@ -238,6 +248,8 @@ extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
 extern bool ismc;		/* Is the program running as a mail client? */
 extern bool isimap;		/* Is the program running as an imap client? */
 extern bool down_bg;		/* download in background */
+extern char whichproc; // which edbrowse-xx process
+extern char showProgress; // feedback as a file is downloaded
 extern int eb_lang;		/* edbrowse language, determined by $LANG */
 extern bool cons_utf8;		/* does the console expect utf8? */
 extern bool iuConvert;		/* perform iso utf8 conversions automatically */
@@ -258,19 +270,23 @@ extern char errorMsg[];		/* generated error message */
 extern char serverLine[];	/* lines to and from the mail server */
 extern int localAccount;	/* this is the smtp server for outgoing mail */
 extern char *mailDir;		/* move to this directory when fetching mail */
-extern char *downDir;		/* the download directory */
 extern char *mailUnread;	/* place to hold fetched but unread mail */
 extern char *mailReply;		/* file to hold reply info for each email */
 /* Keep a copy of unformatted mail that you probably won't need again,
  * but you never know. Should probably live somewhere under .Trash */
 extern char *mailStash;
+extern char *downDir;		/* the download directory */
+extern char *ebTempDir;		/* edbrowse temp, such as /tmp/.edbrowse */
+extern char *ebUserDir;		/* $ebTempDir/nnn user ID appended */
 extern char *dbarea, *dblogin, *dbpw;	/* to log into the database */
 extern bool fetchBlobColumns;
 extern bool caseInsensitive, searchStringsAll;
 extern bool allowRedirection;	/* from http code 301, or http refresh */
 extern bool sendReferrer;	/* in the http header */
 extern bool allowJS;		/* javascript on */
+extern bool allowXHR;		/* xhr on */
 extern bool htmlGenerated;
+extern bool ftpActive;
 extern bool helpMessagesOn;	/* no need to type h */
 extern bool pluginsOn;		/* plugins are active */
 extern bool showHiddenFiles;	/* during directory scan */
@@ -371,6 +387,7 @@ struct ebWindow {
 	bool undoable:1;	/* undo is possible */
 	bool sqlMode:1;		/* accessing a table */
 	bool baseset:1; /* <base> tag seen */
+	bool f_encoded:1; /* filename is url encoded */
 	char *dw;		/* document.write string */
 	int dw_l;		/* length of the above */
 /* The javascript context and window corresponding to this edbrowse buffer.
@@ -494,7 +511,7 @@ enum {
 	TAGACT_MUSIC, TAGACT_IMAGE, TAGACT_BR, TAGACT_IBR, TAGACT_P,
 	TAGACT_BASE, TAGACT_META, TAGACT_PRE, TAGACT_TBODY,
 	TAGACT_DT, TAGACT_DD, TAGACT_LI, TAGACT_TABLE, TAGACT_TR, TAGACT_TD,
-	TAGACT_DIV, TAGACT_SPAN, TAGACT_HR,
+	TAGACT_DIV, TAGACT_SPAN, TAGACT_HR, TAGACT_OBJECT,
 	TAGACT_FORM, TAGACT_FRAME,
 	TAGACT_MAP, TAGACT_AREA, TAGACT_SCRIPT, TAGACT_EMBED, TAGACT_OBJ,
 };
