@@ -49,7 +49,7 @@ static void markAllDead(void)
 
 /* communication pipes with the js process */
 static int pipe_in[2], pipe_out[2];
-static char arg1[8], arg2[8], arg3[8];
+static char arg1[8], arg2[8];
 
 static int js_pid;
 static struct EJ_MSG head;
@@ -63,47 +63,19 @@ static struct EJ_MSG head;
 static pthread_t tid;
 static void *child_proc(void *vp)
 {
+	int rc;
 /* child here, exec the back end js process */
-	size_t len;
-// change [d:\foo\bar\]edbrowse[.exe] to [d:\foo\bar\]edbrowse-js[.exe]
-	char drive[_MAX_DRIVE];
-	char dir[_MAX_DIR];
-	char fname[_MAX_FNAME];
-	char ext[_MAX_EXT];
-	char pn[MAX_PATH];
-	// split it up
-	_splitpath(progname, drive, dir, fname, ext);
-	// put it back together
-	pn[0] = 0;
-	strcat(pn, drive);
-	strcat(pn, dir);
-	len = strlen(fname);
-	if (len && (fname[len - 1] == 'd')) {
-		fname[len - 1] = 0;
-		strcat(pn, fname);
-		strcat(pn, "-jsd");	// add to the debug file name
-	} else {
-		strcat(pn, fname);
-		strcat(pn, "-js");	// add to the file name
-	}
-
-	strcat(pn, ext);
-
 	//close(pipe_in[0]);
 	//close(pipe_out[1]);
 	sprintf(arg1, "%d", pipe_out[0]);
 	sprintf(arg2, "%d", pipe_in[1]);
-	sprintf(arg3, "%d", jsPool);
-
-	debugPrint(5, "spawning '%s' %s %s %s", pn, arg1, arg2, arg3);
-	len = _spawnl(_P_WAIT, pn, "edbrowse-js", arg1, arg2, arg3, 0);
-	//_execlp(pn, "edbrowse-js", arg1, arg2, arg3, 0);
-	// len = 1;
-	if (len) {
+	debugPrint(5, "spawning '%s' %s %s", progname, arg1, arg2);
+	rc = _spawnl(_P_WAIT, progname, "edbrowse", "--mode", "js", arg1, arg2,
+		     0);
+	if (rc) {
 		debugPrint(5, "spawning FAILED! %d\n", errno);
 /* oops, process did not exec */
 /* write a message from this child, saying js would not exec */
-/* The process just started; head is zero */
 		head.magic = EJ_MAGIC;
 		head.highstat = EJ_HIGH_PROC_FAIL;
 		head.lowstat = EJ_LOW_EXEC;
@@ -112,7 +84,6 @@ static void *child_proc(void *vp)
 	}
 	return (void *)90;
 }
-
 #endif // defined(DOSLIKE) && defined(HAVE_PTHREAD_H)
 
 /* Start the js process. */
@@ -121,9 +92,6 @@ static void js_start(void)
 	int pid;
 	char *jsprog;
 
-	if (js_pid)
-		return;		/* already running */
-
 #if defined(DOSLIKE) && !defined(HAVE_PTHREAD_H)
 	debugPrint(5,
 		   "no pthread, so no communication channels for javascript");
@@ -192,32 +160,17 @@ static void js_start(void)
 	close(pipe_out[1]);
 	sprintf(arg1, "%d", pipe_out[0]);
 	sprintf(arg2, "%d", pipe_in[1]);
-	sprintf(arg3, "%d", jsPool);
-	debugPrint(5, "spawning edbrowse-js %s %s %s", arg1, arg2, arg3);
-
-	if (!strchr(progname, '/')) {
-// no path specified, just the program, so assume edbrowse-js is also on $PATH,
-// hopefully in the same bin.
-		execlp("edbrowse-js", "edbrowse-js", arg1, arg2, arg3, NULL);
-	} else {
-// change /foo/bar/edbrowse to /foo/bar/edbrowse-js
-		int l = strlen(progname);
-		char *jspath = allocMem(l + 4);
-		sprintf(jspath, "%s-js", progname);
-		execl(jspath, "edbrowse-js", arg1, arg2, arg3, NULL);
-		nzFree(jspath);
-	}
+	debugPrint(5, "spawning edbrowse-js %s %s", arg1, arg2);
+	execlp(progname, "edbrowse", "--mode", "js", arg1, arg2, NULL);
 
 /* oops, process did not exec */
 /* write a message from this child, saying js would not exec */
-/* The process just started; head is zero */
 	head.magic = EJ_MAGIC;
 	head.highstat = EJ_HIGH_PROC_FAIL;
 	head.lowstat = EJ_LOW_EXEC;
 	write(pipe_in[1], &head, sizeof(head));
 	exit(90);
 #endif // defined(DOSLIKE) && defined(HAVE_PTHREAD_H) y/n
-
 }				/* js_start */
 
 /* Shut down the js process, although if we got here,
@@ -478,8 +431,9 @@ static int readMessage(void)
 			return -1;
 		}
 		effects[head.side] = 0;
-		if (debugLevel >= 4)
-			printf("< side effects\n%s", effects);
+// because debugPrint always puts on a newline
+		effects[head.side - 1] = 0;
+		debugPrint(4, "< side effects\n%s", effects);
 		processEffects();
 	}
 
@@ -494,10 +448,15 @@ static int readMessage(void)
 		msg[l] = 0;
 		if (debugLevel >= 3) {
 /* print message, this will be in English, and mostly for our debugging */
-			if (jsSourceFile)
-				printf("%s line %d: ", jsSourceFile,
-				       head.lineno);
-			printf("%s\n", msg);
+			if (jsSourceFile) {
+				if (debugFile)
+					fprintf(debugFile, "%s line %d: ",
+						jsSourceFile, head.lineno);
+				else
+					printf("%s line %d: ",
+					       jsSourceFile, head.lineno);
+			}
+			debugPrint(3, "%s", msg);
 		}
 		free(msg);
 	}
@@ -558,17 +517,26 @@ static const char *debugString(const char *v)
  * from the js process. */
 static void ack5(void)
 {
+	char *a;
+	int a_l;
+	char buf[32];
 	if (debugLevel < 5)
 		return;
-	printf("<");
-	if (head.highstat)
-		printf(" error %d|%d", head.highstat, head.lowstat);
+	a = initString(&a_l);
+	stringAndChar(&a, &a_l, '<');
+	if (head.highstat) {
+		sprintf(buf, " error %d|%d", head.highstat, head.lowstat);
+		stringAndString(&a, &a_l, buf);
+	}
+	stringAndChar(&a, &a_l, ' ');
 	if (propval)
-		printf(" %s\n", debugString(propval));
+		stringAndString(&a, &a_l, debugString(propval));
 	else if (head.cmd == EJ_CMD_HASPROP)
-		printf(" %d\n", head.proptype);
+		stringAndNum(&a, &a_l, head.proptype);
 	else
-		puts(" ok");
+		stringAndString(&a, &a_l, "ok");
+	debugPrint(5, "%s", a);
+	nzFree(a);
 }				/* ack5 */
 
 /* Create a js context for the current window.
@@ -578,7 +546,10 @@ void createJavaContext(void)
 	if (!allowJS)
 		return;
 
-	js_start();
+	if (!js_pid) {
+		js_start();
+		update_var_in_js(1);
+	}
 
 	debugPrint(5, "> create context for session %d", context);
 
@@ -633,6 +604,8 @@ void freeJavaContext(struct ebWindow *w)
 
 void js_shutdown(void)
 {
+	if (whichproc == 'j')
+		return;
 	if (!js_pid)		/* js not running */
 		return;
 	debugPrint(5, "> js shutdown");
@@ -647,6 +620,8 @@ void js_shutdown(void)
 /* After fork, the child process does not need to talk to js */
 void js_disconnect(void)
 {
+	if (whichproc == 'j')
+		return;
 	if (!js_pid)
 		return;
 	close(pipe_in[0]);
@@ -711,6 +686,8 @@ void jsRunScript(jsobjtype obj, const char *str, const char *filename,
 /* does the member exist? */
 enum ej_proptype has_property(jsobjtype obj, const char *name)
 {
+	if (whichproc == 'j')
+		return has_property_nat(obj, name);
 	if (!allowJS || !cw->winobj || !obj)
 		return EJ_PROP_NONE;
 
@@ -731,6 +708,10 @@ enum ej_proptype has_property(jsobjtype obj, const char *name)
 
 void delete_property(jsobjtype obj, const char *name)
 {
+	if (whichproc == 'j') {
+		delete_property_nat(obj, name);
+		return;
+	}
 	if (!allowJS || !cw->winobj || !obj)
 		return;
 
@@ -775,6 +756,8 @@ static int get_property(jsobjtype obj, const char *name)
 char *get_property_string(jsobjtype obj, const char *name)
 {
 	char *s;
+	if (whichproc == 'j')
+		return get_property_string_nat(obj, name);
 	get_property(obj, name);
 	s = propval;
 	propval = 0;
@@ -825,6 +808,8 @@ bool get_property_bool(jsobjtype obj, const char *name)
 jsobjtype get_property_object(jsobjtype parent, const char *name)
 {
 	jsobjtype child = 0;
+	if (whichproc == 'j')
+		return get_property_object_nat(parent, name);
 	get_property(parent, name);
 	if (!propval)
 		return child;
@@ -871,6 +856,8 @@ static int get_array_element(jsobjtype obj, int idx)
 jsobjtype get_array_element_object(jsobjtype obj, int idx)
 {
 	jsobjtype p = 0;
+	if (whichproc == 'j')
+		return get_array_element_object_nat(obj, idx);
 	get_array_element(obj, idx);
 	if (!propval)
 		return p;
@@ -914,6 +901,8 @@ static int set_property(jsobjtype obj, const char *name,
 
 int set_property_string(jsobjtype obj, const char *name, const char *value)
 {
+	if (whichproc == 'j')
+		return set_property_string_nat(obj, name, value);
 	if (value == NULL)
 		value = emptyString;
 	return set_property(obj, name, value, EJ_PROP_STRING);
@@ -922,6 +911,8 @@ int set_property_string(jsobjtype obj, const char *name, const char *value)
 int set_property_number(jsobjtype obj, const char *name, int n)
 {
 	char buf[20];
+	if (whichproc == 'j')
+		return set_property_number_nat(obj, name, n);
 	sprintf(buf, "%d", n);
 	return set_property(obj, name, buf, EJ_PROP_INT);
 }				/* set_property_number */
@@ -929,6 +920,8 @@ int set_property_number(jsobjtype obj, const char *name, int n)
 int set_property_float(jsobjtype obj, const char *name, double n)
 {
 	char buf[32];
+	if (whichproc == 'j')
+		return set_property_float_nat(obj, name, n);
 	sprintf(buf, "%lf", n);
 	return set_property(obj, name, buf, EJ_PROP_FLOAT);
 }				/* set_property_float */
@@ -936,6 +929,8 @@ int set_property_float(jsobjtype obj, const char *name, double n)
 int set_property_bool(jsobjtype obj, const char *name, bool n)
 {
 	char buf[8];
+	if (whichproc == 'j')
+		return set_property_bool_nat(obj, name, n);
 	strcpy(buf, (n ? "1" : "0"));
 	return set_property(obj, name, buf, EJ_PROP_BOOL);
 }				/* set_property_bool */
@@ -943,6 +938,8 @@ int set_property_bool(jsobjtype obj, const char *name, bool n)
 int set_property_object(jsobjtype parent, const char *name, jsobjtype child)
 {
 	char buf[32];
+	if (whichproc == 'j')
+		return set_property_object_nat(parent, name, child);
 	sprintf(buf, "%p", child);
 	return set_property(parent, name, buf, EJ_PROP_OBJECT);
 }				/* set_property_object */
@@ -951,6 +948,9 @@ jsobjtype instantiate_array(jsobjtype parent, const char *name)
 {
 	jsobjtype p = 0;
 
+	if (whichproc == 'j')
+		return instantiate_array_nat(parent, name);
+
 	if (!allowJS || !cw->winobj || !parent)
 		return 0;
 
@@ -1009,6 +1009,8 @@ static int set_array_element(jsobjtype array, int idx,
 int set_array_element_object(jsobjtype array, int idx, jsobjtype child)
 {
 	char buf[32];
+	if (whichproc == 'j')
+		return set_array_element_object_nat(array, idx, child);
 	sprintf(buf, "%p", child);
 	return set_array_element(array, idx, buf, EJ_PROP_OBJECT);
 }				/* set_array_element_object */
@@ -1017,6 +1019,8 @@ jsobjtype instantiate_array_element(jsobjtype array, int idx,
 				    const char *classname)
 {
 	jsobjtype p = 0;
+	if (whichproc == 'j')
+		return instantiate_array_element_nat(array, idx, classname);
 	set_array_element(array, idx, classname, EJ_PROP_INSTANCE);
 	if (!propval)
 		return p;
@@ -1033,6 +1037,9 @@ jsobjtype instantiate(jsobjtype parent, const char *name, const char *classname)
 {
 	jsobjtype p = 0;
 
+	if (whichproc == 'j')
+		return instantiate_nat(parent, name, classname);
+
 	if (!allowJS || !cw->winobj || !parent)
 		return 0;
 
@@ -1067,6 +1074,8 @@ jsobjtype instantiate(jsobjtype parent, const char *name, const char *classname)
 
 int set_property_function(jsobjtype parent, const char *name, const char *body)
 {
+	if (whichproc == 'j')
+		return set_property_function_nat(parent, name, body);
 	if (!body)
 		body = emptyString;
 	return set_property(parent, name, body, EJ_PROP_FUNCTION);
@@ -1119,6 +1128,8 @@ static int run_function(jsobjtype obj, const char *name, int argc,
 
 int get_arraylength(jsobjtype a)
 {
+	if (whichproc == 'j')
+		return get_arraylength_nat(a);
 	head.cmd = EJ_CMD_ARLEN;
 	head.obj = a;
 	if (writeHeader())
@@ -1129,6 +1140,22 @@ int get_arraylength(jsobjtype a)
 	return head.n;
 }				/* get_arraylength */
 
+/* A global variable has changed that js needs to know about. */
+void update_var_in_js(int varid)
+{
+	int value = 0;
+	if (!js_pid)
+		return;
+	if (varid == 1)
+		value = allowXHR;
+	debugPrint(5, "> varupdate %d", varid);
+	head.cmd = EJ_CMD_VARUPDATE;
+	head.obj = 0;
+	head.lineno = varid;
+	head.n = value;
+	writeHeader();
+}				/* update_var_in_js */
+
 /*********************************************************************
 Everything beyond this point is, perhaps, part of a DOM support layer
 above what has come before.
@@ -1585,5 +1612,9 @@ void run_function_objargs(jsobjtype obj, const char *name, int nargs, ...)
 
 void run_function_onearg(jsobjtype obj, const char *name, jsobjtype a)
 {
+	if (whichproc == 'j') {
+		run_function_onearg_nat(obj, name, a);
+		return;
+	}
 	run_function_objargs(obj, name, 1, a);
 }				/* run_function_onearg */
