@@ -13,7 +13,6 @@ bool cxCompare(int cx) ;
 bool cxActive(int cx) ;
 bool cxQuit(int cx, int action) ;
 void cxSwitch(int cx, bool interactive) ;
-void gotoLocation(char *url, int delay, bool rf) ;
 bool addTextToBuffer(const pst inbuf, int length, int destl, bool onside) ;
 void delText(int start, int end) ;
 bool readFileArgv(const char *filename);
@@ -80,6 +79,7 @@ jsobjtype instantiate_array_element(jsobjtype array, int idx, const char *classn
 jsobjtype instantiate(jsobjtype parent, const char *name, const char *classname) ;
 int set_property_function(jsobjtype parent, const char *name, const char *body) ;
 int get_arraylength(jsobjtype a);
+void update_var_in_js(int varid);
 char *get_property_option(jsobjtype obj) ;
 void setupJavaDom(void) ;
 char *get_property_url(jsobjtype owner, bool action) ;
@@ -95,7 +95,7 @@ int fetchMail(int account) ;
 int fetchAllMail(void) ;
 void scanMail(void) ;
 bool emailTest(void) ;
-void unpackUploadedFile(const char *post, const char *boundary, char **postb, int *postb_l) ;
+void mail64Error(int err);
 char *emailParse(char *buf) ;
 bool setupReply(bool all) ;
 
@@ -109,9 +109,7 @@ void extractEmailAddresses(char *line) ;
 void cutDuplicateEmails(char *tolist, char *cclist, const char *reply) ;
 bool looksBinary(const char *buf, int buflen) ;
 void looks_8859_utf8(const char *buf, int buflen, bool * iso_p, bool * utf8_p) ;
-uchar base64Bits(char c);
 char *base64Encode(const char *inbuf, int inlen, bool lines);
-int base64Decode(char *start, char **end);
 void iuReformat(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p) ;
 bool parseDataURI(const char *uri, char **mediatype, char **data, int *data_l);
 
@@ -161,30 +159,35 @@ void htmlNodesIntoTree(int start, struct htmlTag *attach);
 void html_from_setter( jsobjtype innerParent, const char *h);
 
 /* sourcefile=http.c */
+void gotoLocation(char *url, int delay, bool rf) ;
 size_t eb_curl_callback(char *incoming, size_t size, size_t nitems, struct eb_curl_callback_data *data) ;
+uchar base64Bits(char c);
+int base64Decode(char *start, char **end);
 char *extractHeaderParam(const char *str, const char *item) ;
 time_t parseHeaderDate(const char *date) ;
 bool parseRefresh(char *ref, int *delay_p) ;
 bool refreshDelay(int sec, const char *u) ;
-bool httpConnect(const char *url, bool down_ok, bool webpage);
+bool httpConnect(const char *url, bool down_ok, bool webpage, bool f_encoded, char **headers_p, char **body_p, int *bodlen_p);
 void ebcurl_setError(CURLcode curlret, const char *url) ;
 void setHTTPLanguage(const char *lang) ;
-void http_curl_init(void) ;
 int ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data, size_t size, void *unused) ;
 int bg_jobs(bool iponly);
 void addNovsHost(char *host) ;
+void deleteNovsHosts(void);
 CURLcode setCurlURL(CURL * h, const char *url) ;
 const char *findProxyForURL(const char *url) ;
 
 /* sourcefile=main.c */
 const char *mailRedirect(const char *to, const char *from, const char *reply, const char *subj) ;
 bool javaOK(const char *url) ;
+void eb_curl_global_init(void);
 void ebClose(int n) ;
 bool isSQL(const char *s) ;
 void setDataSource(char *v) ;
 bool runEbFunction(const char *line) ;
 struct DBTABLE *findTableDescriptor(const char *sn) ;
 struct DBTABLE *newTableDescriptor(const char *name) ;
+bool readConfigFile(void);
 
 /* sourcefile=plugin.c */
 const struct MIMETYPE *findMimeBySuffix(const char *suffix) ;
@@ -278,6 +281,7 @@ bool isA(char c) ;
 bool isquote(char c) ;
 void errorPrint(const char *msg, ...) ;
 void debugPrint(int lev, const char *msg, ...) ;
+void setDebugFile(const char *name);
 void nl(void) ;
 int perl2c(char *t) ;
 unsigned pstLength(pst s) ;
@@ -319,6 +323,7 @@ int eb_system(const char *cmd, bool print_on_success);
 void unpercentURL(char *url) ;
 void unpercentString(char *s) ;
 char *percentURL(const char *start, const char *end);
+bool looksPercented(const char *start, const char *end);
 char *htmlEscape0(const char *s, bool do_and);
 #define htmlEscape(s) htmlEscape0((s), true)
 #define htmlEscapeTextarea(s) htmlEscape0((s), false)
@@ -344,3 +349,24 @@ char *encodePostData(const char *s) ;
 char *decodePostData(const char *data, const char *name, int seqno) ;
 void decodeMailURL(const char *url, char **addr_p, char **subj_p, char **body_p) ;
 
+/* sourcefile=jseng-moz.cpp */
+int js_main(int argc, char **argv);
+// the native versions of the api functions in ebjs.c
+enum ej_proptype has_property_nat(jsobjtype obj, const char *name) ;
+void delete_property_nat(jsobjtype obj, const char *name) ;
+char *get_property_string_nat(jsobjtype obj, const char *name) ;
+jsobjtype get_property_object_nat(jsobjtype parent, const char *name) ;
+jsobjtype get_array_element_object_nat(jsobjtype obj, int idx) ;
+int set_property_string_nat(jsobjtype obj, const char *name, const char *value) ;
+int set_property_number_nat(jsobjtype obj, const char *name, int n) ;
+int set_property_float_nat(jsobjtype obj, const char *name, double n) ;
+int set_property_bool_nat(jsobjtype obj, const char *name, bool n) ;
+int set_property_object_nat(jsobjtype parent, const char *name, jsobjtype child) ;
+jsobjtype instantiate_array_nat(jsobjtype parent, const char *name) ;
+int set_array_element_object_nat(jsobjtype array, int idx, jsobjtype child) ;
+jsobjtype instantiate_array_element_nat(jsobjtype array, int idx, const char *classname) ;
+jsobjtype instantiate_nat(jsobjtype parent, const char *name, const char *classname) ;
+int set_property_function_nat(jsobjtype parent, const char *name, const char *body) ;
+int get_arraylength_nat(jsobjtype a);
+void run_function_onearg_nat(jsobjtype obj, const char *name, jsobjtype o);
+
