@@ -331,6 +331,7 @@ static void undosOneMessage(void)
 
 static char presentMail(void);
 static void envelopes(CURL * handle, struct FOLDER *f);
+static void isoDecode(char *vl, char **vrp);
 
 static void cleanFolder(struct FOLDER *f)
 {
@@ -458,6 +459,9 @@ static void scanFolder(CURL * handle, struct FOLDER *f, bool allmessages)
 	char cust_cmd[80];
 	char inputline[80];
 	bool yesdel = false, delflag;
+	char *envp;		// print the envelope concisely
+	char *envp_end;
+	int envp_l;
 
 	if (!f->nmsgs || !allmessages && !f->unread) {
 		i_puts(MSG_NoMessages);
@@ -487,12 +491,25 @@ showmessages:
 		if (!allmessages && mif->seen)
 			continue;
 
+// Print the envelope. From and subject could be encoded in various ways.
+		envp = initString(&envp_l);
 		if (!mif->seen)
-			printf("*");
-		printf("%s: %s", mif->from, mif->subject);
-		if (mif->sent)
-			printf(" %s", conciseTime(mif->sent));
-		printf(" %s\n", conciseSize(mif->size));
+			stringAndChar(&envp, &envp_l, '*');
+		stringAndString(&envp, &envp_l, mif->from);
+		stringAndString(&envp, &envp_l, ": ");
+		stringAndString(&envp, &envp_l, mif->subject);
+		if (mif->sent) {
+			stringAndChar(&envp, &envp_l, ' ');
+			stringAndString(&envp, &envp_l, conciseTime(mif->sent));
+		}
+		stringAndChar(&envp, &envp_l, ' ');
+		stringAndString(&envp, &envp_l, conciseSize(mif->size));
+		envp_end = envp + envp_l;
+		isoDecode(envp, &envp_end);
+		*envp_end = 0;
+// Resulting line could contain utf8, use the portable puts routine.
+		eb_puts(envp);
+		nzFree(envp);
 
 action:
 		delflag = false;
@@ -1397,7 +1414,7 @@ saveMail:
 		write(fh,
 		      "======================================================================\n",
 		      71);
-	if (key == 'u') {
+	if (key == 'u' || key == 'U') {
 		if (write(fh, mailstring, mailstring_l) < mailstring_l) {
 badsave:
 			i_printf(MSG_NoWrite, atname);
@@ -1408,8 +1425,8 @@ badsave:
 		fsize = mailstring_l;
 	} else {
 
-/* key = w, write the file - if pop then save the original unformatted */
-		if (!isimap && mailStash) {
+/* key = w, write the file and save the original unformatted */
+		if (mailStash) {
 			char *rmf;	/* raw mail file */
 			int rmfh;	/* file handle to same */
 /* I want a fairly easy filename, in case I want to go look at the original.
@@ -1602,7 +1619,7 @@ bool emailTest(void)
 	return false;
 }				/* emailTest */
 
-static void mail64Error(int err)
+void mail64Error(int err)
 {
 	switch (err) {
 	case BAD_BASE64_DECODE:
@@ -1643,53 +1660,6 @@ static void unpackQP(struct MHINFO *w)
 	*r = 0;
 }				/* unpackQP */
 
-void
-unpackUploadedFile(const char *post, const char *boundary,
-		   char **postb, int *postb_l)
-{
-	static const char message64[] = "Content-Transfer-Encoding: base64";
-	const int boundlen = strlen(boundary);
-	const int m64len = strlen(message64);
-	char *post2;
-	char *b1, *b2, *b3, *b4;	/* boundary points */
-	int unpack_ret;
-
-	*postb = 0;
-	*postb_l = 0;
-	if (!strstr(post, message64))
-		return;
-
-	post2 = cloneString(post);
-	b2 = strstr(post2, boundary);
-	while (true) {
-		b1 = b2 + boundlen;
-		if (*b1 != '\r')
-			break;
-		b1 += 2;
-		b1 = strstr(b1, "Content-Transfer");
-		b2 = strstr(b1, boundary);
-		if (memcmp(b1, message64, m64len))
-			continue;
-		b1 += m64len - 6;
-		strcpy(b1, "8bit\r\n\r\n");
-		b1 += 8;
-		b1[0] = b1[1] = ' ';
-		b3 = b2 - 4;
-
-		b4 = b3;
-		unpack_ret = base64Decode(b1, &b4);
-		if (unpack_ret != GOOD_BASE64_DECODE)
-			mail64Error(unpack_ret);
-		/* Should we *really* keep going at this point? */
-		strmove(b4, b3);
-		b2 = b4 + 4;
-	}
-
-	b1 += strlen(b1);
-	*postb = post2;
-	*postb_l = b1 - post2;
-}				/* unpackUploadedFile */
-
 /* Look for the name of the attachment and boundary */
 static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 {
