@@ -393,34 +393,36 @@ static int idxperiod, idxcomma, idxright, idxany;
 static void debugChunk(const char *chunk, int len)
 {
 	int i;
+	FILE *f;
 	if (debugLevel < 7)
 		return;
-	printf("chunk<");
+	f = debugFile ? debugFile : stdout;
+	fprintf(f, "chunk<");
 	for (i = 0; i < len; ++i) {
 		char c = chunk[i];
 		if (c == '\t') {
-			printf("\\t");
+			fprintf(f, "\\t");
 			continue;
 		}
 		if (c == '\n') {
-			printf("\\n");
+			fprintf(f, "\\n");
 			continue;
 		}
 		if (c == '\f') {
-			printf("\\f");
+			fprintf(f, "\\f");
 			continue;
 		}
 		if (c == '\r') {
-			printf("\\r");
+			fprintf(f, "\\r");
 			continue;
 		}
 		if (c == '\0') {
-			printf("\\0");
+			fprintf(f, "\\0");
 			continue;
 		}
-		printf("%c", c);
+		fprintf(f, "%c", c);
 	}
-	printf(">%d.%d\n", colno, lspace);
+	fprintf(f, ">%d.%d\n", colno, lspace);
 }				/* debugChunk */
 
 static void appendOneChar(char c)
@@ -1141,83 +1143,6 @@ char *base64Encode(const char *inbuf, int inlen, bool lines)
 	return outstr;
 }				/* base64Encode */
 
-uchar base64Bits(char c)
-{
-	if (isupperByte(c))
-		return c - 'A';
-	if (islowerByte(c))
-		return c - ('a' - 26);
-	if (isdigitByte(c))
-		return c - ('0' - 52);
-	if (c == '+')
-		return 62;
-	if (c == '/')
-		return 63;
-	return 64;		/* error */
-}				/* base64Bits */
-
-/*
- * Decode some data in base64.
- * This function operates on the data in-line.  It does not allocate a fresh
- * string to hold the decoded data.  Since the data will be smaller than
- * the base64 encoded representation, this cannot overflow buffers.
- * If you need to preserve the input, copy it first.
- *
- * start points to the start of the input
- * *end initially points to the byte just after the end of the input
- * Returns: GOOD_BASE64_DECODE on success, BAD_BASE64_DECODE or
- * EXTRA_CHARS_BASE64_DECODE on error.
- * When the function returns success, *end points to the end of the decoded
- * data.  On failure, end points to the just past the end of
- * what was successfully decoded. */
-int base64Decode(char *start, char **end)
-{
-	char *b64_end = *end;
-	uchar val, leftover, mod;
-	bool equals;
-	int ret = GOOD_BASE64_DECODE;
-	char c, *q, *r;
-/* Since this is a copy, and the unpacked version is always
- * smaller, just unpack it inline. */
-	mod = 0;
-	equals = false;
-	for (q = r = start; q < b64_end; ++q) {
-		c = *q;
-		if (isspaceByte(c))
-			continue;
-		if (equals) {
-			if (c == '=')
-				continue;
-			ret = EXTRA_CHARS_BASE64_DECODE;
-			break;
-		}
-		if (c == '=') {
-			equals = true;
-			continue;
-		}
-		val = base64Bits(c);
-		if (val & 64) {
-			ret = BAD_BASE64_DECODE;
-			break;
-		}
-		if (mod == 0) {
-			leftover = val << 2;
-		} else if (mod == 1) {
-			*r++ = (leftover | (val >> 4));
-			leftover = val << 4;
-		} else if (mod == 2) {
-			*r++ = (leftover | (val >> 2));
-			leftover = val << 6;
-		} else {
-			*r++ = (leftover | val);
-		}
-		++mod;
-		mod &= 3;
-	}
-	*end = r;
-	return ret;
-}				/* base64Decode */
-
 void
 iuReformat(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
 {
