@@ -12,39 +12,32 @@
 #endif
 #include <time.h>
 
-CURL *http_curl_handle = NULL;
 char *serverData;
 int serverDataLen;
-static int down_fd;		/* downloading file descriptor */
-static const char *down_file;	/* downloading filename */
-static const char *down_file2;	/* without download directory */
+CURL *global_http_handle;
+CURLSH *global_share_handle;
+bool pluginsOn = true;
+bool down_bg;			/* download in background */
+char showProgress = 'd';	// dots
+
+static CURL *down_h;
 static int down_pid;		/* pid of the downloading child process */
 static bool down_permitted;
-/* download states.
- * -1 user aborted the download
- * 0 in-memory download
- * 1 download but stop and ask user if it looks binary
-* 2 disk download foreground
-* 3 disk download background parent
-* 4 disk download background child
-* 5 disk download background prefork
- * 6 mime type says this should be a stream */
-static int down_state;
-bool down_bg;			/* download in background */
-static int down_length;		/* how much data to disk so far */
 static int down_msg;
 
 struct BG_JOB {
 	struct BG_JOB *next, *prev;
 	int pid, state;
+	size_t fsize;		// file size
+	int file2;		// offset into filename
 	char file[4];
 };
 struct listHead down_jobs = {
 	&down_jobs, &down_jobs
 };
 
-static void background_download(void);
-static void setup_download(void);
+static void background_download(struct eb_curl_callback_data *data);
+static void setup_download(struct eb_curl_callback_data *data);
 static char errorText[CURL_ERROR_SIZE + 1];
 static char *http_headers;
 static int http_headers_len;
@@ -54,10 +47,27 @@ static char *httpLanguage;
 static char hct[60];
 static char *hct2;		/* extra content info such as charset */
 int hcl;			/* http content length */
+static CURL *http_curl_init(struct eb_curl_callback_data *cbd);
 
-static struct eb_curl_callback_data callback_data = {
-	&serverData, &serverDataLen
-};
+/* This function is called for web redirection, by the refresh command,
+ * or by window.location = new_url. */
+char *newlocation;
+int newloc_d;			/* possible delay */
+bool newloc_r;			/* replace the buffer */
+bool js_redirects;
+void gotoLocation(char *url, int delay, bool rf)
+{
+	if (newlocation && delay >= newloc_d) {
+		nzFree(url);
+		return;
+	}
+	nzFree(newlocation);
+	newlocation = url;
+	newloc_d = delay;
+	newloc_r = rf;
+	if (!delay)
+		js_redirects = true;
+}				/* gotoLocation */
 
 /* string is allocated. Quotes are removed. No other processing is done.
  * You may need to decode %xx bytes or such. */
@@ -126,7 +136,8 @@ static void scan_http_headers(bool fromCallback)
 	if (!hcl && (v = find_http_header("content-length"))) {
 		hcl = atoi(v);
 		nzFree(v);
-		debugPrint(3, "content length %d", hcl);
+		if (hcl)
+			debugPrint(3, "content length %d", hcl);
 	}
 
 	if (fromCallback)
@@ -151,9 +162,10 @@ static void scan_http_headers(bool fromCallback)
 }				/* scan_http_headers */
 
 /* actually run the curl request, http or ftp or whatever */
-static CURLcode fetch_internet(bool is_http)
+static CURLcode fetch_internet(CURL * h, bool is_http)
 {
 	CURLcode curlret;
+	down_h = h;
 /* this should already be 0 */
 	nzFree(newlocation);
 	newlocation = NULL;
@@ -162,21 +174,29 @@ static CURLcode fetch_internet(bool is_http)
 	hct[0] = 0;
 	hct2 = NULL;
 	hcl = 0;
-	curlret = curl_easy_perform(http_curl_handle);
+	curlret = curl_easy_perform(h);
 	if (is_http)
 		scan_http_headers(false);
-	nzFree(http_headers);
-	http_headers = 0;
 	return curlret;
 }				/* fetch_internet */
 
-static bool ftpConnect(const char *url, const char *user, const char *pass);
+static bool ftpConnect(const char *url, const char *user, const char *pass,
+		       bool f_encoded);
 static bool read_credentials(char *buffer);
 static size_t curl_header_callback(char *header_line, size_t size, size_t nmemb,
-				   void *unused);
+				   struct eb_curl_callback_data *data);
 static const char *message_for_response_code(int code);
 
-/* Callback used by libcurl. Appends data to serverData. */
+/* Callback used by libcurl. Captures data from http, ftp, pop3, etc.
+ * download states:
+ * -1 user aborted the download
+ * 0 standard in-memory download
+ * 1 download but stop and ask user if he wants to download to disk
+* 2 disk download foreground
+* 3 disk download background parent
+* 4 disk download background child
+* 5 disk download background prefork
+ * 6 mime type says this should be a stream */
 size_t
 eb_curl_callback(char *incoming, size_t size, size_t nitems,
 		 struct eb_curl_callback_data *data)
@@ -184,27 +204,47 @@ eb_curl_callback(char *incoming, size_t size, size_t nitems,
 	size_t num_bytes = nitems * size;
 	int dots1, dots2, rc;
 
-	if (down_state == 1) {
+	if (data->down_state == 1) {
+		if (hcl == 0) {
+// http should always set http content length, this is just for ftp.
+// And ftp downloading a file always has state = 1 on the first data block.
+			double d_size = 0.0;	// download size, if we can get it
+			curl_easy_getinfo(down_h,
+					  CURLINFO_CONTENT_LENGTH_DOWNLOAD,
+					  &d_size);
+			hcl = d_size;
+			if (hcl < 0)
+				hcl = 0;
+		}
+
 /* state 1, first data block, ask the user */
-		setup_download();
-		if (down_state == 0)
+		setup_download(data);
+		if (data->down_state == 0)
 			goto showdots;
-		if (down_state == -1 || down_state == 5)
+		if (data->down_state == -1 || data->down_state == 5)
 			return -1;
 	}
 
-	if (down_state == 2 || down_state == 4) {	/* to disk */
-		rc = write(down_fd, incoming, num_bytes);
+	if (data->down_state == 2 || data->down_state == 4) {	/* to disk */
+		rc = write(data->down_fd, incoming, num_bytes);
 		if (rc == num_bytes) {
-			if (down_state == 4)
+			if (data->down_state == 4) {
+#if 0
+// Deliberately delay background download, to get several running in parallel
+// for testing purposes.
+				if (data->down_length == 0)
+					sleep(17);
+				data->down_length += rc;
+#endif
 				return rc;
+			}
 			goto showdots;
 		}
-		if (down_state == 2) {
-			setError(MSG_NoWrite2, down_file);
+		if (data->down_state == 2) {
+			setError(MSG_NoWrite2, data->down_file);
 			return -1;
 		}
-		i_printf(MSG_NoWrite2, down_file);
+		i_printf(MSG_NoWrite2, data->down_file);
 		printf(", ");
 		i_puts(MSG_DownAbort);
 		exit(1);
@@ -212,15 +252,20 @@ eb_curl_callback(char *incoming, size_t size, size_t nitems,
 
 showdots:
 	dots1 = *(data->length) / CHUNKSIZE;
-	if (down_state == 0)
+	if (data->down_state == 0)
 		stringAndBytes(data->buffer, data->length, incoming, num_bytes);
 	else
 		*(data->length) += num_bytes;
 	dots2 = *(data->length) / CHUNKSIZE;
-	if (dots1 < dots2) {
-		for (; dots1 < dots2; ++dots1)
-			putchar('.');
-		fflush(stdout);
+	if (showProgress != 'q' && dots1 < dots2) {
+		if (showProgress == 'd') {
+			for (; dots1 < dots2; ++dots1)
+				putchar('.');
+			fflush(stdout);
+		}
+		if (showProgress == 'c' && hcl)
+			printf("%d/%d\n", dots2,
+			       (hcl + CHUNKSIZE - 1) / CHUNKSIZE);
 	}
 	return num_bytes;
 }
@@ -251,6 +296,131 @@ curl_progress(void *unused, double dl_total, double dl_now,
 	return ret;
 }				/* curl_progress */
 
+uchar base64Bits(char c)
+{
+	if (isupperByte(c))
+		return c - 'A';
+	if (islowerByte(c))
+		return c - ('a' - 26);
+	if (isdigitByte(c))
+		return c - ('0' - 52);
+	if (c == '+')
+		return 62;
+	if (c == '/')
+		return 63;
+	return 64;		/* error */
+}				/* base64Bits */
+
+/*********************************************************************
+Decode some data in base64.
+This function operates on the data in-line.  It does not allocate a fresh
+string to hold the decoded data.  Since the data will be smaller than
+the base64 encoded representation, this cannot overflow.
+If you need to preserve the input, copy it first.
+start points to the start of the input
+*end initially points to the byte just after the end of the input
+Returns: GOOD_BASE64_DECODE on success, BAD_BASE64_DECODE or
+EXTRA_CHARS_BASE64_DECODE on error.
+When the function returns success, *end points to the end of the decoded
+data.  On failure, end points to the just past the end of
+what was successfully decoded.
+*********************************************************************/
+
+int base64Decode(char *start, char **end)
+{
+	char *b64_end = *end;
+	uchar val, leftover, mod;
+	bool equals;
+	int ret = GOOD_BASE64_DECODE;
+	char c, *q, *r;
+/* Since this is a copy, and the unpacked version is always
+ * smaller, just unpack it inline. */
+	mod = 0;
+	equals = false;
+	for (q = r = start; q < b64_end; ++q) {
+		c = *q;
+		if (isspaceByte(c))
+			continue;
+		if (equals) {
+			if (c == '=')
+				continue;
+			ret = EXTRA_CHARS_BASE64_DECODE;
+			break;
+		}
+		if (c == '=') {
+			equals = true;
+			continue;
+		}
+		val = base64Bits(c);
+		if (val & 64) {
+			ret = BAD_BASE64_DECODE;
+			break;
+		}
+		if (mod == 0) {
+			leftover = val << 2;
+		} else if (mod == 1) {
+			*r++ = (leftover | (val >> 4));
+			leftover = val << 4;
+		} else if (mod == 2) {
+			*r++ = (leftover | (val >> 2));
+			leftover = val << 6;
+		} else {
+			*r++ = (leftover | val);
+		}
+		++mod;
+		mod &= 3;
+	}
+	*end = r;
+	return ret;
+}				/* base64Decode */
+
+static void
+unpackUploadedFile(const char *post, const char *boundary,
+		   char **postb, int *postb_l)
+{
+	static const char message64[] = "Content-Transfer-Encoding: base64";
+	const int boundlen = strlen(boundary);
+	const int m64len = strlen(message64);
+	char *post2;
+	char *b1, *b2, *b3, *b4;	/* boundary points */
+	int unpack_ret;
+
+	*postb = 0;
+	*postb_l = 0;
+	if (!strstr(post, message64))
+		return;
+
+	post2 = cloneString(post);
+	b2 = strstr(post2, boundary);
+	while (true) {
+		b1 = b2 + boundlen;
+		if (*b1 != '\r')
+			break;
+		b1 += 2;
+		b1 = strstr(b1, "Content-Transfer");
+		b2 = strstr(b1, boundary);
+		if (memcmp(b1, message64, m64len))
+			continue;
+		b1 += m64len - 6;
+		strcpy(b1, "8bit\r\n\r\n");
+		b1 += 8;
+		b1[0] = b1[1] = ' ';
+		b3 = b2 - 4;
+
+		b4 = b3;
+		unpack_ret = base64Decode(b1, &b4);
+		if (unpack_ret != GOOD_BASE64_DECODE)
+			mail64Error(unpack_ret);
+		/* Should we *really* keep going at this point? */
+		strmove(b4, b3);
+		b2 = b4 + 4;
+	}
+
+	b1 += strlen(b1);
+	*postb = post2;
+	*postb_l = b1 - post2;
+}				/* unpackUploadedFile */
+
 /* Pull a keyword: attribute out of an internet header. */
 static char *extractHeaderItem(const char *head, const char *end,
 			       const char *item, const char **ptr)
@@ -528,9 +698,49 @@ bool refreshDelay(int sec, const char *u)
 long hcode;			/* example, 404 */
 static char herror[32];		/* example, file not found */
 static char *urlcopy;
+static int urlcopy_l;
 
-bool httpConnect(const char *url, bool down_ok, bool webpage)
+// encode the url, if it was supplied by the user.
+// Otherwise just make a copy.
+// Either way there is room for one more char at the end.
+static void urlSanitize(const char *url, const char *post, bool f_encoded)
 {
+	const char *portloc;
+
+	if (f_encoded && !looksPercented(url, post)) {
+		debugPrint(2, "Warning, url %s doesn't look encoded", url);
+		f_encoded = false;
+	}
+
+	if (!f_encoded) {
+		urlcopy = percentURL(url, post);
+		urlcopy_l = strlen(urlcopy);
+	} else {
+		if (post)
+			urlcopy_l = post - url;
+		else
+			urlcopy_l = strlen(url);
+		urlcopy = allocMem(urlcopy_l + 2);
+		strncpy(urlcopy, url, urlcopy_l);
+		urlcopy[urlcopy_l] = 0;
+	}
+
+// get rid of : in http://this.that.com:/path, curl can't handle it.
+	getPortLocURL(urlcopy, &portloc, 0);
+	if (portloc && !isdigit(portloc[1])) {
+		const char *s = portloc + strcspn(portloc, "/?#\1");
+		strmove((char *)portloc, s);
+		urlcopy_l = strlen(urlcopy);
+	}
+}				/* urlSanitize */
+
+// Last three are result parameters, for http headers and body strings.
+// Set to 0 if you don't want these passed back in this way.
+bool httpConnect(const char *url, bool down_ok, bool webpage,
+		 bool f_encoded, char **headers_p, char **body_p, int *bodlen_p)
+{
+	CURL *h;		// the curl http handle
+	struct eb_curl_callback_data cbd;
 	char *referrer = NULL;
 	CURLcode curlret = CURLE_OK;
 	struct curl_slist *custom_headers = NULL;
@@ -550,6 +760,13 @@ bool httpConnect(const char *url, bool down_ok, bool webpage)
 	int redirect_count = 0;
 	bool name_changed = false;
 
+	if (headers_p)
+		*headers_p = 0;
+	if (body_p)
+		*body_p = 0;
+	if (bodlen_p)
+		*bodlen_p = 0;
+
 	urlcopy = NULL;
 	serverData = NULL;
 	serverDataLen = 0;
@@ -587,7 +804,7 @@ bool httpConnect(const char *url, bool down_ok, bool webpage)
 		   stringEqualCI(prot, "ftps") ||
 		   stringEqualCI(prot, "scp") ||
 		   stringEqualCI(prot, "tftp") || stringEqualCI(prot, "sftp")) {
-		return ftpConnect(url, user, pass);
+		return ftpConnect(url, user, pass, f_encoded);
 	} else if ((cw->mt = findMimeByProtocol(prot)) && pluginsOn
 		   && cw->mt->stream) {
 mimestream:
@@ -606,6 +823,16 @@ mimestream:
 	if ((cw->mt = findMimeByURL(url)) && pluginsOn && cw->mt->stream)
 		goto mimestream;
 
+	cbd.buffer = &serverData;
+	cbd.length = &serverDataLen;
+	cbd.down_state = 0;
+	cbd.down_file = cbd.down_file2 = NULL;
+	cbd.down_length = 0;
+	down_permitted = down_ok;
+	h = http_curl_init(&cbd);
+	if (!h)
+		return false;
+
 /* "Expect:" header causes some servers to lose.  Disable it. */
 	tmp_headers = curl_slist_append(custom_headers, "Expect:");
 	if (tmp_headers == NULL)
@@ -620,8 +847,9 @@ mimestream:
 
 	post = strchr(url, '\1');
 	postb = 0;
+	urlSanitize(url, post, f_encoded);
+
 	if (post) {
-		urlcopy = percentURL(url, post);
 		post++;
 
 		if (strncmp(post, "`mfd~", 5)) ;	/* No need to do anything, not multipart. */
@@ -652,19 +880,17 @@ mimestream:
 			post = s + 2;
 			unpackUploadedFile(post, thisbound, &postb, &postb_l);
 		}
-		curlret = curl_easy_setopt(http_curl_handle, CURLOPT_POSTFIELDS,
+		curlret = curl_easy_setopt(h, CURLOPT_POSTFIELDS,
 					   (postb_l ? postb : post));
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 		curlret =
-		    curl_easy_setopt(http_curl_handle, CURLOPT_POSTFIELDSIZE,
+		    curl_easy_setopt(h, CURLOPT_POSTFIELDSIZE,
 				     postb_l ? postb_l : strlen(post));
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 	} else {
-		urlcopy = percentURL(url, NULL);
-		curlret =
-		    curl_easy_setopt(http_curl_handle, CURLOPT_HTTPGET, 1);
+		curlret = curl_easy_setopt(h, CURLOPT_HTTPGET, 1);
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 	}
@@ -682,15 +908,13 @@ mimestream:
 		referrer = cw->referrer;
 	}
 
-	curlret = curl_easy_setopt(http_curl_handle, CURLOPT_REFERER, referrer);
+	curlret = curl_easy_setopt(h, CURLOPT_REFERER, referrer);
 	if (curlret != CURLE_OK)
 		goto curl_fail;
-	curlret =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_HTTPHEADER,
-			     custom_headers);
+	curlret = curl_easy_setopt(h, CURLOPT_HTTPHEADER, custom_headers);
 	if (curlret != CURLE_OK)
 		goto curl_fail;
-	curlret = setCurlURL(http_curl_handle, urlcopy);
+	curlret = setCurlURL(h, urlcopy);
 	if (curlret != CURLE_OK)
 		goto curl_fail;
 
@@ -710,8 +934,7 @@ mimestream:
  * If the URL didn't have user and password, and getUserPass failed,
  * then creds_buf == "".
  */
-	curlret =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_USERPWD, creds_buf);
+	curlret = curl_easy_setopt(h, CURLOPT_USERPWD, creds_buf);
 	if (curlret != CURLE_OK)
 		goto curl_fail;
 
@@ -732,55 +955,57 @@ mimestream:
 
 	while (still_fetching == true) {
 		char *redir = NULL;
-		down_state = 0;
-		down_file = NULL;
-		down_permitted = down_ok;
-		callback_data.length = &serverDataLen;
+		cbd.length = &serverDataLen;
 
 perform:
-		curlret = fetch_internet(true);
+		curlret = fetch_internet(h, true);
 
-		if (down_state == 6)
+		if (cbd.down_state == 6) {
+			curl_easy_cleanup(h);
 			goto mimestream;
+		}
 
-		if (down_state == 5) {
+		if (cbd.down_state == 5) {
 /* user has directed a download of this file in the background. */
-			background_download();
-			if (down_state == 4)
+			background_download(&cbd);
+			if (cbd.down_state == 4)
 				goto perform;
 		}
 
-		if (down_state == 3 || down_state == -1) {
+		if (cbd.down_state == 3 || cbd.down_state == -1) {
 /* set this to null so we don't push a new buffer */
 			serverData = NULL;
+			cnzFree(cbd.down_file);
+			curl_easy_cleanup(h);
 			return false;
 		}
 
-		if (down_state == 4) {
+		if (cbd.down_state == 4) {
 			if (curlret != CURLE_OK) {
 				ebcurl_setError(curlret, urlcopy);
 				showError();
 				exit(2);
 			}
 			i_printf(MSG_DownSuccess);
-			printf(": %s\n", down_file2);
+			printf(": %s\n", cbd.down_file2);
 			exit(0);
 		}
 
-		if (*(callback_data.length) >= CHUNKSIZE)
-			nl();	/* We printed dots, so we terminate them with newline */
+		if (*(cbd.length) >= CHUNKSIZE && showProgress == 'd')
+			nl();	/* We printed dots, so terminate them with newline */
 
-		if (down_state == 2) {
-			close(down_fd);
+		if (cbd.down_state == 2) {
+			close(cbd.down_fd);
+			cnzFree(cbd.down_file);
 			setError(MSG_DownSuccess);
 			serverData = NULL;
+			curl_easy_cleanup(h);
 			return false;
 		}
 
 		if (curlret != CURLE_OK)
 			goto curl_fail;
-		curl_easy_getinfo(http_curl_handle, CURLINFO_RESPONSE_CODE,
-				  &hcode);
+		curl_easy_getinfo(h, CURLINFO_RESPONSE_CODE, &hcode);
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 
@@ -820,21 +1045,19 @@ perform:
 /* Convert POST request to GET request after redirection. */
 /* This should only be done for 301 through 303 */
 				if (hcode < 307)
-					curl_easy_setopt(http_curl_handle,
-							 CURLOPT_HTTPGET, 1);
+					curl_easy_setopt(h, CURLOPT_HTTPGET, 1);
 /* I think there is more work to do for 307 308,
  * pasting the prior post string onto the new URL. Not sure about this. */
 
 				getUserPass(urlcopy, creds_buf, false);
 
 				curlret =
-				    curl_easy_setopt(http_curl_handle,
-						     CURLOPT_USERPWD,
+				    curl_easy_setopt(h, CURLOPT_USERPWD,
 						     creds_buf);
 				if (curlret != CURLE_OK)
 					goto curl_fail;
 
-				curlret = setCurlURL(http_curl_handle, urlcopy);
+				curlret = setCurlURL(h, urlcopy);
 				if (curlret != CURLE_OK)
 					goto curl_fail;
 
@@ -855,8 +1078,7 @@ perform:
 			got_creds = read_credentials(creds_buf);
 			if (got_creds) {
 				addWebAuthorization(urlcopy, creds_buf, false);
-				curl_easy_setopt(http_curl_handle,
-						 CURLOPT_USERPWD, creds_buf);
+				curl_easy_setopt(h, CURLOPT_USERPWD, creds_buf);
 				nzFree(serverData);
 				serverData = emptyString;
 				serverDataLen = 0;
@@ -864,9 +1086,7 @@ perform:
 /* User aborted the login process, try and at least get something. */
 				proceed_unauthenticated = true;
 			}
-		}
-		/* authenticate? */
-		else {		/* not redirect, not 401 */
+		} else {	/* not redirect, not 401 */
 			still_fetching = false;
 			transfer_status = true;
 		}
@@ -875,6 +1095,9 @@ perform:
 curl_fail:
 	if (custom_headers)
 		curl_slist_free_all(custom_headers);
+// Don't need the handle any more.
+	curl_easy_cleanup(h);
+
 	if (curlret != CURLE_OK)
 		ebcurl_setError(curlret, urlcopy);
 
@@ -890,8 +1113,8 @@ curl_fail:
 		if (hcode != 200 && hcode != 201 &&
 		    (webpage || debugLevel >= 2) ||
 		    hcode == 201 && debugLevel >= 3)
-			i_printf(MSG_HTTPError, hcode,
-				 message_for_response_code(hcode));
+			i_printf(MSG_HTTPError,
+				 hcode, message_for_response_code(hcode));
 		if (name_changed)
 			changeFileName = urlcopy;
 		else
@@ -910,6 +1133,21 @@ curl_fail:
 		return rc;
 	}
 
+	if (transfer_status) {
+		if (headers_p) {
+			*headers_p = http_headers;
+// The string is your responsibility now.
+			http_headers = 0;
+		}
+		if (body_p) {
+			*body_p = serverData;
+			*bodlen_p = serverDataLen;
+// The string is your responsibility now.
+			serverData = 0;
+			serverDataLen = 0;
+		}
+	}
+
 	return transfer_status;
 }				/* httpConnect */
 
@@ -1128,17 +1366,18 @@ void ebcurl_setError(CURLcode curlret, const char *url)
 }				/* ebcurl_setError */
 
 /* Like httpConnect, but for ftp */
-static bool ftpConnect(const char *url, const char *user, const char *pass)
+static bool ftpConnect(const char *url, const char *user, const char *pass,
+		       bool f_encoded)
 {
+	CURL *h;		// the curl handle for ftp
+	struct eb_curl_callback_data cbd;
 	int protLength;		/* length of "ftp://" */
-	int urlcopy_l = 0;
 	bool transfer_success = false;
 	bool has_slash, is_scp;
 	CURLcode curlret = CURLE_OK;
 	char creds_buf[MAXUSERPASS * 2 + 1];
 	size_t creds_len = 0;
 
-	urlcopy = NULL;
 	protLength = strchr(url, ':') - url + 3;
 /* scp is somewhat unique among the protocols handled here */
 	is_scp = memEqualCI(url, "scp", 3);
@@ -1152,69 +1391,78 @@ static bool ftpConnect(const char *url, const char *user, const char *pass)
 		strcpy(creds_buf, "anonymous:ftp@example.com");
 	}
 
-	curlret =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_USERPWD, creds_buf);
+	cbd.buffer = &serverData;
+	cbd.length = &serverDataLen;
+	cbd.down_state = 0;
+	cbd.down_file = cbd.down_file2 = NULL;
+	cbd.down_length = 0;
+	h = http_curl_init(&cbd);
+	if (!h)
+		goto ftp_transfer_fail;
+	curlret = curl_easy_setopt(h, CURLOPT_USERPWD, creds_buf);
 	if (curlret != CURLE_OK)
 		goto ftp_transfer_fail;
 
 	serverData = initString(&serverDataLen);
-	urlcopy = initString(&urlcopy_l);
-	stringAndString(&urlcopy, &urlcopy_l, url);
+	urlSanitize(url, 0, f_encoded);
 
 /* libcurl appends an implicit slash to URLs like "ftp://foo.com".
 * Be explicit, so that edbrowse knows that we have a directory. */
 
 	if (!strchr(urlcopy + protLength, '/'))
-		stringAndChar(&urlcopy, &urlcopy_l, '/');
+		strcpy(urlcopy + urlcopy_l, "/");
 
-	curlret = setCurlURL(http_curl_handle, urlcopy);
+	curlret = setCurlURL(h, urlcopy);
 	if (curlret != CURLE_OK)
 		goto ftp_transfer_fail;
 
-	has_slash = urlcopy[urlcopy_l - 1] == '/';
+	has_slash = urlcopy[urlcopy_l] == '/';
 /* don't download a directory listing, we want to see that */
 /* Fetching a directory will fail in the special case of scp. */
-	down_state = (has_slash ? 0 : 1);
-	down_file = NULL;	/* should already be null */
+	cbd.down_state = (has_slash ? 0 : 1);
 	down_msg = MSG_FTPDownload;
 	if (is_scp)
 		down_msg = MSG_SCPDownload;
-	callback_data.length = &serverDataLen;
+	cbd.length = &serverDataLen;
 
 perform:
-	curlret = fetch_internet(false);
+	curlret = fetch_internet(h, false);
 
-	if (down_state == 5) {
+	if (cbd.down_state == 5) {
 /* user has directed a download of this file in the background. */
-		background_download();
-		if (down_state == 4)
+		background_download(&cbd);
+		if (cbd.down_state == 4)
 			goto perform;
 	}
 
-	if (down_state == 3 || down_state == -1) {
+	if (cbd.down_state == 3 || cbd.down_state == -1) {
 /* set this to null so we don't push a new buffer */
 		serverData = NULL;
+		cnzFree(cbd.down_file);
+		curl_easy_cleanup(h);
 		return false;
 	}
 
-	if (down_state == 4) {
+	if (cbd.down_state == 4) {
 		if (curlret != CURLE_OK) {
 			ebcurl_setError(curlret, urlcopy);
 			showError();
 			exit(2);
 		}
 		i_printf(MSG_DownSuccess);
-		printf(": %s\n", down_file2);
+		printf(": %s\n", cbd.down_file2);
 		exit(0);
 	}
 
-	if (*(callback_data.length) >= CHUNKSIZE)
-		nl();		/* We printed dots, so we terminate them with newline */
+	if (*(cbd.length) >= CHUNKSIZE && showProgress == 'd')
+		nl();		/* We printed dots, so terminate them with newline */
 
-	if (down_state == 2) {
-		close(down_fd);
+	if (cbd.down_state == 2) {
+		close(cbd.down_fd);
 		setError(MSG_DownSuccess);
 		serverData = NULL;
+		cnzFree(cbd.down_file);
+		curl_easy_cleanup(h);
 		return false;
 	}
 
@@ -1225,13 +1473,15 @@ perform:
 		if (has_slash | is_scp)
 			transfer_success = false;
 		else {		/* try appending a slash. */
-			stringAndChar(&urlcopy, &urlcopy_l, '/');
-			down_state = 0;
-			curlret = setCurlURL(http_curl_handle, urlcopy);
+			strcpy(urlcopy + urlcopy_l, "/");
+			cbd.down_state = 0;
+			cnzFree(cbd.down_file);
+			cbd.down_file = 0;
+			curlret = setCurlURL(h, urlcopy);
 			if (curlret != CURLE_OK)
 				goto ftp_transfer_fail;
 
-			curlret = fetch_internet(false);
+			curlret = fetch_internet(h, false);
 			if (curlret != CURLE_OK)
 				transfer_success = false;
 			else {
@@ -1247,6 +1497,9 @@ perform:
 		transfer_success = false;
 
 ftp_transfer_fail:
+// Don't need the handle any more
+	if (h)
+		curl_easy_cleanup(h);
 	if (transfer_success == false) {
 		if (curlret != CURLE_OK)
 			ebcurl_setError(curlret, urlcopy);
@@ -1268,6 +1521,11 @@ void setHTTPLanguage(const char *lang)
 {
 	int httpLanguage_l;
 
+	nzFree(httpLanguage);
+	httpLanguage = NULL;
+	if (!lang)
+		return;
+
 	httpLanguage = initString(&httpLanguage_l);
 	stringAndString(&httpLanguage, &httpLanguage_l, "Accept-Language: ");
 	stringAndString(&httpLanguage, &httpLanguage_l, lang);
@@ -1294,39 +1552,41 @@ my_curl_safeSocket(void *clientp, curl_socket_t socketfd, curlsocktype purpose)
 #endif // _MSC_VER y/n
 }
 
-/* Clean up libcurl's state. */
-static void my_curl_cleanup(void)
-{
-	curl_easy_cleanup(http_curl_handle);
-	curl_global_cleanup();
-}				/* my_curl_cleanup */
-
-void http_curl_init(void)
+static CURL *http_curl_init(struct eb_curl_callback_data *cbd)
 {
 	CURLcode curl_init_status = CURLE_OK;
-	http_curl_handle = curl_easy_init();
-	if (http_curl_handle == NULL)
-		i_printfExit(MSG_LibcurlNoInit);
-
+	CURL *h = curl_easy_init();
+	if (h == NULL)
+		goto libcurl_init_fail;
+	curl_init_status =
+	    curl_easy_setopt(h, CURLOPT_SHARE, global_share_handle);
+	if (curl_init_status != CURLE_OK)
+		goto libcurl_init_fail;
 /* Lots of these setopt calls shouldn't fail.  They just diddle a struct. */
-	curl_easy_setopt(http_curl_handle, CURLOPT_SOCKOPTFUNCTION,
-			 my_curl_safeSocket);
-	curl_easy_setopt(http_curl_handle, CURLOPT_WRITEFUNCTION,
-			 eb_curl_callback);
-	curl_easy_setopt(http_curl_handle, CURLOPT_WRITEDATA, &callback_data);
-	curl_easy_setopt(http_curl_handle, CURLOPT_HEADERFUNCTION,
-			 curl_header_callback);
+	curl_easy_setopt(h, CURLOPT_SOCKOPTFUNCTION, my_curl_safeSocket);
+	curl_easy_setopt(h, CURLOPT_WRITEFUNCTION, eb_curl_callback);
+	curl_easy_setopt(h, CURLOPT_WRITEDATA, cbd);
+	curl_easy_setopt(h, CURLOPT_HEADERFUNCTION, curl_header_callback);
+	curl_easy_setopt(h, CURLOPT_HEADERDATA, cbd);
 	if (debugLevel >= 4)
-		curl_easy_setopt(http_curl_handle, CURLOPT_VERBOSE, 1);
-	curl_easy_setopt(http_curl_handle, CURLOPT_DEBUGFUNCTION,
-			 ebcurl_debug_handler);
-	curl_easy_setopt(http_curl_handle, CURLOPT_NOPROGRESS, 0);
-	curl_easy_setopt(http_curl_handle, CURLOPT_PROGRESSFUNCTION,
-			 curl_progress);
-	curl_easy_setopt(http_curl_handle, CURLOPT_CONNECTTIMEOUT, webTimeout);
-	curl_easy_setopt(http_curl_handle, CURLOPT_USERAGENT, currentAgent);
-	curl_easy_setopt(http_curl_handle, CURLOPT_SSLVERSION,
-			 CURL_SSLVERSION_DEFAULT);
+		curl_easy_setopt(h, CURLOPT_VERBOSE, 1);
+	curl_easy_setopt(h, CURLOPT_DEBUGFUNCTION, ebcurl_debug_handler);
+	curl_easy_setopt(h, CURLOPT_NOPROGRESS, 0);
+	curl_easy_setopt(h, CURLOPT_PROGRESSFUNCTION, curl_progress);
+	curl_easy_setopt(h, CURLOPT_CONNECTTIMEOUT, webTimeout);
+	curl_easy_setopt(h, CURLOPT_USERAGENT, currentAgent);
+	curl_easy_setopt(h, CURLOPT_SSLVERSION, CURL_SSLVERSION_DEFAULT);
+	curl_easy_setopt(h, CURLOPT_USERAGENT, currentAgent);
+/* We're doing this manually for now.
+	curl_easy_setopt(h, CURLOPT_FOLLOWLOCATION, allowRedirection);
+*/
+	curl_easy_setopt(h, CURLOPT_AUTOREFERER, sendReferrer);
+	if (ftpActive)
+		curl_easy_setopt(h, CURLOPT_FTPPORT, "-");
+	else
+		curl_easy_setopt(h, CURLOPT_FTPPORT, NULL);
+/* See "man curl_easy_setopt.3" for info on CURLOPT_FTPPORT.  Supplying
+* "-" makes libcurl select the best IP address for active ftp. */
 
 /*
 * tell libcurl to pick the strongest method from basic, digest and ntlm authentication
@@ -1334,36 +1594,24 @@ void http_curl_init(void)
 * and it looks like in most cases microsoft IIS says it supports both and libcurl
 * doesn't fall back to NTLM when it discovers that Negotiate isn't set up on a system
 */
-	curl_easy_setopt(http_curl_handle, CURLOPT_HTTPAUTH,
+	curl_easy_setopt(h, CURLOPT_HTTPAUTH,
 			 CURLAUTH_BASIC | CURLAUTH_DIGEST | CURLAUTH_NTLM);
 
 /* The next few setopt calls could allocate or perform file I/O. */
 	errorText[0] = '\0';
-	curl_init_status =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_ERRORBUFFER, errorText);
+	curl_init_status = curl_easy_setopt(h, CURLOPT_ERRORBUFFER, errorText);
 	if (curl_init_status != CURLE_OK)
 		goto libcurl_init_fail;
-	curl_init_status =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_CAINFO, sslCerts);
+	curl_init_status = curl_easy_setopt(h, CURLOPT_ENCODING, "");
 	if (curl_init_status != CURLE_OK)
 		goto libcurl_init_fail;
-	curl_init_status =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_COOKIEFILE, cookieFile);
-	if (curl_init_status != CURLE_OK)
-		goto libcurl_init_fail;
-	curl_init_status =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_COOKIEJAR, cookieFile);
-	if (curl_init_status != CURLE_OK)
-		goto libcurl_init_fail;
-	curl_init_status =
-	    curl_easy_setopt(http_curl_handle, CURLOPT_ENCODING, "");
-	if (curl_init_status != CURLE_OK)
-		goto libcurl_init_fail;
-	atexit(my_curl_cleanup);
+	return h;
 
 libcurl_init_fail:
-	if (curl_init_status != CURLE_OK)
-		i_printfExit(MSG_LibcurlNoInit);
+	i_printf(MSG_LibcurlNoInit);
+	if (h)
+		curl_easy_cleanup(h);
+	return 0;
 }				/* http_curl_init */
 
 /*
@@ -1544,25 +1792,26 @@ static bool read_credentials(char *buffer)
 /* Callback used by libcurl.
  * Gather all the http headers into one long string. */
 static size_t
-curl_header_callback(char *header_line, size_t size, size_t nmemb, void *unused)
+curl_header_callback(char *header_line, size_t size, size_t nmemb,
+		     struct eb_curl_callback_data *data)
 {
 	size_t bytes_in_line = size * nmemb;
 	stringAndBytes(&http_headers, &http_headers_len,
 		       header_line, bytes_in_line);
 
-	if (down_permitted && down_state == 0 && !hct[0]) {
-		scan_http_headers(true);
+	scan_http_headers(true);
+	if (down_permitted && data->down_state == 0) {
 		if (cw->mt && cw->mt->stream && pluginsOn) {
 /* I don't think this ever happens, since streams are indicated by the protocol,
  * and we wouldn't even get here, but just in case -
  * stop the download and set the flag so we can pass this url
  * to the program that handles this kind of stream. */
-			down_state = 6;
+			data->down_state = 6;
 			return -1;
 		}
 		if (hct[0] && !memEqualCI(hct, "text/", 5) &&
 		    (!pluginsOn || !cw->mt || cw->mt->download)) {
-			down_state = 1;
+			data->down_state = 1;
 			down_msg = MSG_Down;
 		}
 	}
@@ -1591,14 +1840,15 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 		     size_t size, void *unused)
 {
 	static bool last_curlin = false;
+	FILE *f = debugFile ? debugFile : stdout;
 
 	if (info_desc == CURLINFO_HEADER_OUT) {
-		printf("curl>\n");
-		prettify_network_text(data, size, stdout);
+		fprintf(f, "curl>\n");
+		prettify_network_text(data, size, f);
 	} else if (info_desc == CURLINFO_HEADER_IN) {
 		if (!last_curlin)
-			printf("curl<\n");
-		prettify_network_text(data, size, stdout);
+			fprintf(f, "curl<\n");
+		prettify_network_text(data, size, f);
 	} else;			/* Do nothing.  We don't care about this piece of data. */
 
 	if (info_desc == CURLINFO_HEADER_IN)
@@ -1610,15 +1860,14 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 }				/* ebcurl_debug_handler */
 
 /* At this point, down_state = 1 */
-static void setup_download(void)
+static void setup_download(struct eb_curl_callback_data *data)
 {
 	const char *filepart;
 	const char *answer;
-	int msg;
 
 /* if not run from a terminal then just return. */
-	if (!isatty(0)) {
-		down_state = 0;
+	if (!isInteractive) {
+		data->down_state = 0;
 		return;
 	}
 
@@ -1627,12 +1876,12 @@ top:
 	answer = getFileName(down_msg, filepart, false, true);
 /* space for a filename means read into memory */
 	if (stringEqual(answer, " ")) {
-		down_state = 0;	/* in memory download */
+		data->down_state = 0;	/* in memory download */
 		return;
 	}
 
 	if (stringEqual(answer, "x") || stringEqual(answer, "X")) {
-		down_state = -1;
+		data->down_state = -1;
 		setError(MSG_DownAbort);
 		return;
 	}
@@ -1642,31 +1891,31 @@ top:
 		goto top;
 	}
 
-	down_fd = creat(answer, 0666);
-	if (down_fd < 0) {
+	data->down_fd = creat(answer, 0666);
+	if (data->down_fd < 0) {
 		i_printf(MSG_NoCreate2, answer);
 		nl();
 		goto top;
 	}
-
-	down_file = down_file2 = answer;
+// free down_file, but not down_file2
+	data->down_file = data->down_file2 = cloneString(answer);
 	if (downDir) {
 		int l = strlen(downDir);
-		if (!strncmp(down_file2, downDir, l)) {
-			down_file2 += l;
-			if (down_file2[0] == '/')
-				++down_file2;
+		if (!strncmp(data->down_file2, downDir, l)) {
+			data->down_file2 += l;
+			if (data->down_file2[0] == '/')
+				++data->down_file2;
 		}
 	}
-	down_state = (down_bg ? 5 : 2);
-	callback_data.length = &down_length;
+	data->down_state = (down_bg ? 5 : 2);
+	data->length = &data->down_length;
 }				/* setup_download */
 
 #ifdef _MSC_VER			// need fork()
 /* At this point, down_state = 5 */
-static void background_download(void)
+static void background_download(struct eb_curl_callback_data *data)
 {
-	down_state = -1;
+	data->down_state = -1;
 /* perhaps a better error message here */
 	setError(MSG_DownAbort);
 	return;
@@ -1680,11 +1929,11 @@ int bg_jobs(bool iponly)
 #else // !_MSC_VER
 
 /* At this point, down_state = 5 */
-static void background_download(void)
+static void background_download(struct eb_curl_callback_data *data)
 {
 	down_pid = fork();
 	if (down_pid < 0) {	/* should never happen */
-		down_state = -1;
+		data->down_state = -1;
 /* perhaps a better error message here */
 		setError(MSG_DownAbort);
 		return;
@@ -1692,16 +1941,20 @@ static void background_download(void)
 
 	if (down_pid) {		/* parent */
 		struct BG_JOB *job;
-		close(down_fd);
+		close(data->down_fd);
 /* the error message here isn't really an error, but a progress message */
 		setError(MSG_DownProgress);
-		down_state = 3;
+		data->down_state = 3;
 
-/* push job onto the list, for tracking and display */
-		job = allocMem(sizeof(struct BG_JOB) + strlen(down_file2));
+/* push job onto the list for tracking and display */
+		job = allocMem(sizeof(struct BG_JOB) + strlen(data->down_file));
 		job->pid = down_pid;
 		job->state = 4;
-		strcpy(job->file, down_file2);
+		strcpy(job->file, data->down_file);
+		job->file2 = data->down_file2 - data->down_file;
+// round file size up to the nearest chunk.
+// This will come out 0 only if the true size is 0.
+		job->fsize = ((hcl + (CHUNKSIZE - 1)) / CHUNKSIZE);
 		addToListBack(&down_jobs, job);
 
 		return;
@@ -1711,10 +1964,11 @@ static void background_download(void)
 	js_disconnect();
 /* ignore interrupt, not sure about quit and hangup */
 	signal(SIGINT, SIG_IGN);
-	down_state = 4;
+	data->down_state = 4;
 }				/* background_download */
 
 /* show background jobs and return the number of jobs pending */
+/* if iponly is true then just show in progress */
 int bg_jobs(bool iponly)
 {
 	bool present = false, part;
@@ -1738,6 +1992,7 @@ int bg_jobs(bool iponly)
 /* in progress */
 	part = false;
 	foreach(j, down_jobs) {
+		size_t now_size;
 		if (j->state != 4)
 			continue;
 		++numback;
@@ -1746,7 +2001,11 @@ int bg_jobs(bool iponly)
 			puts(" {");
 			part = present = true;
 		}
-		puts(j->file);
+		printf("%s", j->file + j->file2);
+		if (j->fsize)
+			printf(" %d/%d",
+			       (fileSizeByName(j->file) / CHUNKSIZE), j->fsize);
+		nl();
 	}
 	if (part)
 		puts("}");
@@ -1764,7 +2023,7 @@ int bg_jobs(bool iponly)
 			puts(" {");
 			part = present = true;
 		}
-		puts(j->file);
+		puts(j->file + j->file2);
 	}
 	if (part)
 		puts("}");
@@ -1779,7 +2038,7 @@ int bg_jobs(bool iponly)
 			puts(" {");
 			part = present = true;
 		}
-		puts(j->file);
+		puts(j->file + j->file2);
 	}
 	if (part)
 		puts("}");
@@ -1832,6 +2091,13 @@ static bool mustVerifyHost(const char *host)
 	return true;
 }				/* mustVerifyHost */
 
+void deleteNovsHosts(void)
+{
+	nzFree(novs_hosts);
+	novs_hosts = NULL;
+	novs_hosts_avail = novs_hosts_max = 0;
+}				/* deleteNovsHosts */
+
 CURLcode setCurlURL(CURL * h, const char *url)
 {
 	const char *host;
