@@ -32,6 +32,9 @@ Exit codes are as follows:
 #include <limits>
 #include <iostream>
 #include <string>
+#ifdef DOSLIKE
+#include "vsprtf.h"
+#endif // DOSLIKE
 
 /* work around a bug where the standard UINT32_MAX isn't defined, I really hope this is correct */
 #ifndef UINT32_MAX
@@ -44,43 +47,14 @@ Exit codes are as follows:
 
 using namespace std;
 
-/* This function closes edbrowse down, e.g. after malloc failure,
- * it is just a stub for exit here. */
-void ebClose(int n)
-{
-	exit(n);
-}				/* ebclose */
-
-/* stub, this function called by freeTags(), but never called in this context */
-void freeEmptySideBuffer(int n)
-{
-}
-
-/* meta tags don't have side effects from within the js process. */
-void htmlMetaHelper(struct htmlTag *t)
-{
-}
-
-/* textarea does not generate a side buffer here */
-int sideBuffer(int cx, const char *text, int textlen, const char *bufname)
-{
-	return 0;
-}
-
-/* ebrc strings don't mean anything here */
-const char *ebrc_en = emptyString;
-const char *ebrc_fr = emptyString;
-const char *ebrc_pt_br = emptyString;
-const char *ebrc_de = emptyString;
-
 static void usage(void)
 {
-	fprintf(stderr, "Usage:  edbrowse-js pipe_in pipe_out jsHeapSize\n");
+	fprintf(stderr, "Usage:  edbrowse-js pipe_in pipe_out\n");
 	exit(1);
 }				/* usage */
 
 /* arguments, as indicated by the above */
-static int pipe_in, pipe_out, enginePool;
+static int pipe_in, pipe_out;
 
 static void js_start(void);
 static void readMessage(void);
@@ -96,7 +70,7 @@ static void cwSetup(void)
 {
 	in_js_cw.winobj = winobj;
 	in_js_cw.docobj = docobj;
-	in_js_cw.hbase = get_property_string(docobj, "base$href");
+	in_js_cw.hbase = get_property_string_nat(docobj, "base$href");
 	in_js_cw.baseset = true;
 }				/* cwSetup */
 
@@ -139,22 +113,20 @@ static char *propval;
 static enum ej_proptype proptype;
 static char *runscript;
 
-int main(int argc, char **argv)
+int js_main(int argc, char **argv)
 {
-/* do this first, in case usage some day is tailored to language */
-	selectLanguage();
-
-	if (argc != 4)
+	if (argc != 2)
 		usage();
 
-	pipe_in = stringIsNum(argv[1]);
-	pipe_out = stringIsNum(argv[2]);
-	enginePool = stringIsNum(argv[3]);
-	if (pipe_in < 0 || pipe_out < 0 || enginePool < 0)
+	pipe_in = stringIsNum(argv[0]);
+	pipe_out = stringIsNum(argv[1]);
+	if (pipe_in < 0 || pipe_out < 0)
 		usage();
 
-	if (enginePool < 2)
-		enginePool = 2;
+	readConfigFile();
+	eb_curl_global_init();
+	cookiesFromJar();
+	pluginsOn = false;
 
 	js_start();
 
@@ -198,10 +170,19 @@ int main(int argc, char **argv)
 			continue;
 		}
 
+		if (head.cmd == EJ_CMD_VARUPDATE) {
+			if (head.lineno == 1)
+				allowXHR = head.n;
+			head.n = head.proplength = 0;
+//                      no acknowledgement needed
+//                      writeHeader();
+			continue;
+		}
+
 /* this function will enter the compartment */
 		processMessage();
 	}
-}				/* main */
+}				/* js_main */
 
 /* read from and write to edbrowse */
 static void readFromEb(void *data_p, int n)
@@ -315,7 +296,7 @@ static const size_t gStackChunkSize = 8192;
 
 static void js_start(void)
 {
-	jrt = JS_NewRuntime(enginePool * 1024L * 1024L, JS_NO_HELPER_THREADS);
+	jrt = JS_NewRuntime(jsPool * 1024L * 1024L, JS_NO_HELPER_THREADS);
 	if (jrt)
 		return;		/* ok */
 
@@ -585,6 +566,7 @@ generic_class_ctor(document, Document)
     generic_class_ctor(tbody, Tbody)
     generic_class_ctor(table, Table)
     generic_class_ctor(div, Div)
+    generic_class_ctor(htmlobj, HtmlObj)
     generic_class_ctor(area, Area)
     generic_class_ctor(span, Span)
     generic_class_ctor(trow, Trow)
@@ -799,22 +781,22 @@ static ej_proptype find_proptype(JS::HandleObject parent, const char *name)
 	return val_proptype(v);
 }				/* find_proptype */
 
-enum ej_proptype has_property(jsobjtype parent, const char *name)
+enum ej_proptype has_property_nat(jsobjtype parent, const char *name)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	return find_proptype(p, name);
-}				/* has_property */
+}				/* has_property_nat */
 
 static void delete_property1(JS::HandleObject parent, const char *name)
 {
 	JS_DeleteProperty(jcx, parent, name);
 }				/* delete_property1 */
 
-void delete_property(jsobjtype parent, const char *name)
+void delete_property_nat(jsobjtype parent, const char *name)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	delete_property1(p, name);
-}				/* delete_property */
+}				/* delete_property_nat */
 
 static int get_arraylength1(JS::HandleObject a)
 {
@@ -824,11 +806,11 @@ static int get_arraylength1(JS::HandleObject a)
 	return length;
 }				/* get_arraylength1 */
 
-int get_arraylength(jsobjtype a)
+int get_arraylength_nat(jsobjtype a)
 {
 	JS::RootedObject p(jcx, (JSObject *) a);
 	return get_arraylength1(p);
-}				/* get_arraylength */
+}				/* get_arraylength_nat */
 
 /* Use stringize() to return a property as a string, if it is
  * string compatible. The string is allocated, free it when done. */
@@ -853,11 +835,11 @@ static char *get_property_string1(JS::HandleObject parent, const char *name)
 	return cloneString(s);
 }				/* get_property_string1 */
 
-char *get_property_string(jsobjtype parent, const char *name)
+char *get_property_string_nat(jsobjtype parent, const char *name)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	return get_property_string1(p, name);
-}				/* get_property_string */
+}				/* get_property_string_nat */
 
 static JSObject *get_property_object1(JS::HandleObject parent, const char *name)
 {
@@ -871,11 +853,11 @@ static JSObject *get_property_object1(JS::HandleObject parent, const char *name)
 	return child;
 }				/* get_property_object1 */
 
-jsobjtype get_property_object(jsobjtype parent, const char *name)
+jsobjtype get_property_object_nat(jsobjtype parent, const char *name)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	return get_property_object1(p, name);
-}				/* get_property_object */
+}				/* get_property_object_nat */
 
 /* if js changes the value of an input field, this must be reflected
  * in the <foobar> text in edbrowse. */
@@ -1102,12 +1084,13 @@ set_property_string1(js::HandleObject parent, const char *name,
 		misconfigure();
 }				/* set_property_string1 */
 
-int set_property_string(jsobjtype parent, const char *name, const char *value)
+int set_property_string_nat(jsobjtype parent, const char *name,
+			    const char *value)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	set_property_string1(p, name, value);
 	return 0;
-}				/* set_property_string */
+}				/* set_property_string_nat */
 
 static void set_property_bool1(js::HandleObject parent, const char *name,
 			       bool n)
@@ -1129,12 +1112,12 @@ static void set_property_bool1(js::HandleObject parent, const char *name,
 		misconfigure();
 }				/* set_property_bool1 */
 
-int set_property_bool(jsobjtype parent, const char *name, bool n)
+int set_property_bool_nat(jsobjtype parent, const char *name, bool n)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	set_property_bool1(p, name, n);
 	return 0;
-}				/* set_property_bool */
+}				/* set_property_bool_nat */
 
 static void set_property_number1(js::HandleObject parent, const char *name,
 				 int n)
@@ -1156,12 +1139,12 @@ static void set_property_number1(js::HandleObject parent, const char *name,
 		misconfigure();
 }				/* set_property_number1 */
 
-int set_property_number(jsobjtype parent, const char *name, int n)
+int set_property_number_nat(jsobjtype parent, const char *name, int n)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	set_property_number1(p, name, n);
 	return 0;
-}				/* set_property_number */
+}				/* set_property_number_nat */
 
 static void set_property_float1(js::HandleObject parent, const char *name,
 				double n)
@@ -1183,12 +1166,12 @@ static void set_property_float1(js::HandleObject parent, const char *name,
 		misconfigure();
 }				/* set_property_float1 */
 
-int set_property_float(jsobjtype parent, const char *name, double n)
+int set_property_float_nat(jsobjtype parent, const char *name, double n)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	set_property_float1(p, name, n);
 	return 0;
-}				/* set_property_float */
+}				/* set_property_float_nat */
 
 static void
 set_property_object1(js::HandleObject parent, const char *name,
@@ -1215,13 +1198,13 @@ set_property_object1(js::HandleObject parent, const char *name,
 		misconfigure();
 }				/* set_property_object1 */
 
-int set_property_object(jsobjtype parent, const char *name, jsobjtype child)
+int set_property_object_nat(jsobjtype parent, const char *name, jsobjtype child)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	JS::RootedObject c(jcx, (JSObject *) child);
 	set_property_object1(p, name, c);
 	return 0;
-}				/* set_property_object */
+}				/* set_property_object_nat */
 
 /* for window.focus etc */
 static JSBool nullFunction(JSContext * cx, unsigned int argc, jsval * vp)
@@ -1257,12 +1240,13 @@ static void set_property_function1(js::HandleObject parent,
 	}
 }				/* set_property_function1 */
 
-int set_property_function(jsobjtype parent, const char *name, const char *body)
+int set_property_function_nat(jsobjtype parent, const char *name,
+			      const char *body)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	set_property_function1(p, name, body);
 	return 0;
-}				/* set_property_function */
+}				/* set_property_function_nat */
 
 static void
 run_function_onearg1(js::HandleObject parent, const char *name,
@@ -1275,12 +1259,13 @@ run_function_onearg1(js::HandleObject parent, const char *name,
 	JS_CallFunctionName(jcx, parent, name, 1, argv, v.address());
 }				/* run_function_onearg1 */
 
-void run_function_onearg(jsobjtype parent, const char *name, jsobjtype child)
+void run_function_onearg_nat(jsobjtype parent, const char *name,
+			     jsobjtype child)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	JS::RootedObject c(jcx, (JSObject *) child);
 	run_function_onearg1(p, name, c);
-}				/* run_function_onearg */
+}				/* run_function_onearg_nat */
 
 #if 0
 /* Not clear that setAttribute needs any side effects, or needs to be native. */
@@ -1616,6 +1601,107 @@ fail:
 	return JS_TRUE;
 }				/* doc_createElement */
 
+/* Adjust cookie after an http fetch. */
+static void cookieRefresh(void)
+{
+	JS::RootedObject p(jcx);
+	js::RootedValue v(jcx);
+	if (JS_GetProperty(jcx, winobj, "location", v.address()) == JS_TRUE &&
+	    v.isObject()) {
+		p = JSVAL_TO_OBJECT(v);
+		char *url = get_property_string1(p, "href$val");
+		if (url && *url) {
+			int cook_l;
+			char *cook = initString(&cook_l);
+			const char *proto = getProtURL(url);
+			bool secure = false;
+			char *s;
+			if (proto && stringEqualCI(proto, "https"))
+				secure = true;
+			sendCookies(&cook, &cook_l, url, secure);
+			if (memEqualCI(cook, "cookie: ", 8))	/* should often happen */
+				strmove(cook, cook + 8);
+			if (s = strstr(cook, "\r\n"))
+				*s = 0;
+			setter_suspend = true;
+			set_property_string_nat(docobj, "cookie", cook);
+			setter_suspend = false;
+			nzFree(cook);
+		}
+		nzFree(url);
+	}
+}				/* cookieRefresh */
+
+static JSBool fetchHTTP(JSContext * cx, unsigned int argc, jsval * vp)
+{
+	JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
+
+	if (allowXHR) {
+		JS::RootedString incoming_url(cx);
+		JS::RootedString incoming_method(cx);
+		JS::RootedString incoming_headers(cx);
+		JS::RootedString incoming_payload(cx);
+// the order of the parameters is:
+// url, method, headers, payload
+		incoming_url = JS_ValueToString(cx, args[0]);
+		char *curl_incoming_url = JS_c_str(incoming_url);
+		incoming_method = JS_ValueToString(cx, args[1]);
+		char *curl_incoming_method = JS_c_str(incoming_method);
+		incoming_headers = JS_ValueToString(cx, args[2]);
+		char *curl_incoming_headers = JS_c_str(incoming_headers);
+		incoming_payload = JS_ValueToString(cx, args[3]);
+		char *curl_incoming_payload = JS_c_str(incoming_payload);
+
+		char *curl_outgoing_xhrheaders = NULL;
+		char *curl_outgoing_xhrbody = NULL;
+		int responseLength = 0;
+
+		if (curl_incoming_payload && *curl_incoming_payload) {
+			char *a, methchar = '?';
+			if (curl_incoming_method &&
+			    stringEqualCI(curl_incoming_method, "post"))
+				methchar = '\1';
+			if (asprintf(&a, "%s%c%s",
+				     curl_incoming_url, methchar,
+				     curl_incoming_payload) < 0)
+				i_printfExit(MSG_MemAllocError, 50);
+			nzFree(curl_incoming_url);
+			curl_incoming_url = a;
+		}
+
+		httpConnect(curl_incoming_url, false, false, true,
+			    &curl_outgoing_xhrheaders, &curl_outgoing_xhrbody,
+			    &responseLength);
+		args.rval().set(STRING_TO_JSVAL
+				(JS_NewStringCopyZ
+				 (cx,
+				  (string(curl_outgoing_xhrheaders) +
+				   string(curl_outgoing_xhrbody)).c_str())));
+		cnzFree(curl_incoming_url);
+		cnzFree(curl_incoming_method);
+		cnzFree(curl_incoming_headers);
+		cnzFree(curl_incoming_payload);
+		cnzFree(curl_outgoing_xhrheaders);
+		cnzFree(curl_outgoing_xhrbody);
+// http fetch could bring new cookies into the current window.
+// This code commented out and untested because it can't possibly
+// work when the js process and edbrowse have separate cookie spaces.
+// Remember to test this when there is one curl server with one cookie space.
+//              cookieRefresh();
+	} else {
+// current strategy - the send action is allowed,
+// but there will be an empty string returned.
+// So there should not be any runtime errors on account
+// of missing xhr functionality.  The incoming url, etc,
+// are being accepted, just sent to /dev/null unbeknownst
+// to the calling page.  Is this a reasonable way to
+// handle allowXHR == false?
+		args.rval().set(JS_GetEmptyStringValue(cx));
+	}
+
+	return JS_TRUE;
+}
+
 static JSFunctionSpec document_methods[] = {
 	JS_FS("focus", nullFunction, 0, 0),
 	JS_FS("blur", nullFunction, 0, 0),
@@ -1628,6 +1714,7 @@ static JSFunctionSpec document_methods[] = {
 	JS_FS("apch$", apch, 1, 0),
 	JS_FS("insertBefore", insertBefore, 2, 0),
 	JS_FS("removeChild", removeChild, 1, 0),
+	JS_FS("fetchHTTP", fetchHTTP, 4, 0),
 	JS_FS_END
 };
 
@@ -1965,6 +2052,7 @@ static struct {
 	{&trow_class, trow_ctor},
 	{&cell_class, cell_ctor},
 	{&div_class, div_ctor},
+	{&htmlobj_class, htmlobj_ctor},
 	{&area_class, area_ctor},
 	{&span_class, span_ctor},
 	{&option_class, option_ctor, NULL, 2},
@@ -2171,11 +2259,11 @@ static JSObject *instantiate_array1(js::HandleObject parent, const char *name)
 	return a;
 }				/* instantiate_array1 */
 
-jsobjtype instantiate_array(jsobjtype parent, const char *name)
+jsobjtype instantiate_array_nat(jsobjtype parent, const char *name)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	return instantiate_array1(p, name);
-}				/* instantiate_array */
+}				/* instantiate_array_nat */
 
 static JSObject *instantiate1(js::HandleObject parent, const char *name,
 			      const char *classname)
@@ -2203,11 +2291,12 @@ static JSObject *instantiate1(js::HandleObject parent, const char *name,
 	return a;
 }				/* instantiate1 */
 
-jsobjtype instantiate(jsobjtype parent, const char *name, const char *classname)
+jsobjtype instantiate_nat(jsobjtype parent, const char *name,
+			  const char *classname)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	return instantiate1(p, name, classname);
-}				/* instantiate */
+}				/* instantiate_nat */
 
 static JSObject *instantiate_array_element1(js::HandleObject parent, int idx,
 					    const char *classname)
@@ -2219,12 +2308,12 @@ static JSObject *instantiate_array_element1(js::HandleObject parent, int idx,
 	return a;
 }				/* instantiate_array_element1 */
 
-jsobjtype instantiate_array_element(jsobjtype parent, int idx,
-				    const char *classname)
+jsobjtype instantiate_array_element_nat(jsobjtype parent, int idx,
+					const char *classname)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	return instantiate_array_element1(p, idx, classname);
-}				/* instantiate_array_element */
+}				/* instantiate_array_element_nat */
 
 static void
 set_array_element_object1(js::HandleObject parent, int idx,
@@ -2244,7 +2333,7 @@ set_array_element_object1(js::HandleObject parent, int idx,
 	}
 }				/* set_array_element_object1 */
 
-int set_array_element_object(jsobjtype parent, int idx, jsobjtype child)
+int set_array_element_object_nat(jsobjtype parent, int idx, jsobjtype child)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	JS::RootedObject c(jcx, (JSObject *) child);
@@ -2266,11 +2355,11 @@ static JSObject *get_array_element_object1(JS::HandleObject parent, int idx)
 	return child;
 }				/* get_array_element_object1 */
 
-jsobjtype get_array_element_object(jsobjtype parent, int idx)
+jsobjtype get_array_element_object_nat(jsobjtype parent, int idx)
 {
 	JS::RootedObject p(jcx, (JSObject *) parent);
 	return get_array_element_object1(p, idx);
-}				/* get_array_element_object */
+}				/* get_array_element_object_nat */
 
 /*********************************************************************
 ebjs.c allows for geting and setting array elements of all types,
