@@ -431,9 +431,9 @@ enum {
 	MSG_SectionIgnored,
 	MSG_FileXEmpty,
 	MSG_SubjectStart,
-	MSG_notused334,
+	MSG_ProgressQuiet,
 	MSG_SubjectLong,
-	MSG_notused336,
+	MSG_ProgressDots,
 	MSG_SigRegular,
 	MSG_SigAccess,
 	MSG_MailBinary,
@@ -450,7 +450,7 @@ enum {
 	MSG_LineDelete2,
 	MSG_LineAdd1,
 	MSG_LineAdd2,
-	MSG_notused353,
+	MSG_ProgressCount,
 	MSG_MailAccountsNone,
 	MSG_MailAccountBad,
 	MSG_MailBrowse,
@@ -462,7 +462,7 @@ enum {
 	MSG_MailFirstCC,
 	MSG_AttLineX,
 	MSG_MailAccountBadLineX,
-	MSG_notused365,
+	MSG_TempNone,
 	MSG_MailFirstLine,
 	MSG_notused367,
 	MSG_RecipNone3,
@@ -563,8 +563,8 @@ enum {
 	MSG_notused463,
 	MSG_Doubleclick,
 	MSG_OptionSync,
-	MSG_notused466,
-	MSG_notused467,
+	MSG_XhrOff,
+	MSG_XhrOn,
 	MSG_GetLocalJS,
 	MSG_GetJS,
 	MSG_GetJS2,
