@@ -15,7 +15,6 @@ Run audio players, pdf converters, etc, based on suffix or content-type.
  * Since an external program may act upon this file, a certain suffix
  * may be required.
  * Fails if /tmp/.edbrowse does not exist or cannot be created. */
-static const char *tempbase;
 static char *tempin, *tempout;
 static char *suffixin;		/* suffix of input file */
 static char *suffixout;		/* suffix of output file */
@@ -24,41 +23,15 @@ static bool makeTempFilename(const char *suffix, int idx, bool output)
 {
 	char *filename;
 
-	if (!tempbase) {
-/* has not yet been set */
-#ifdef DOSLIKE
-		int l;
-		char *a;
-		tempbase = getenv("TEMP");
-		if (!tempbase) {
-			setError(MSG_NoEnvVar, "TEMP");
-			return false;
-		}
-// put /edbrowse on the end
-		l = strlen(tempbase);
-		a = allocMem(l + 10);
-		sprintf(a, "%s/edbrowse", tempbase);
-		tempbase = a;
-#else
-		tempbase = "/tmp/.edbrowse";
-#endif
-	}
-
-	if (fileTypeByName(tempbase, false) != 'd') {
-/* no such directory, try to make it */
-/* this temp edbrowse directory is used by everyone system wide */
-		if (mkdir(tempbase, 0777)) {
-			setError(MSG_TempDir, tempbase);
-			return false;
-		}
-/* yes, we called mkdir with 777 above, but this call gets us past umask */
-/* What does this do in Windows? */
-		chmod(tempbase, 0777);
+// if no temp directory then we can't proceed
+	if (!ebUserDir) {
+		setError(MSG_TempNone);
+		return false;
 	}
 
 	if (asprintf(&filename, "%s/pf%d-%d.%s",
-		     tempbase, getpid(), idx, suffix) < 0)
-		i_printfExit(MSG_MemAllocError, strlen(tempbase) + 24);
+		     ebUserDir, getpid(), idx, suffix) < 0)
+		i_printfExit(MSG_MemAllocError, strlen(ebUserDir) + 24);
 
 	if (output) {
 // free the last one, don't need it any more.
@@ -199,12 +172,6 @@ char *pluginCommand(const struct MIMETYPE *m,
 
 	len = 0;
 	for (s = m->program; *s; ++s) {
-#if 0
-		if (*s == '*') {
-			len += strlen(suffix);
-			continue;
-		}
-#endif
 		if (*s == '%' && s[1] == 'i') {
 			inlen = shellProtectLength(infile);
 			len += inlen;
@@ -224,13 +191,6 @@ char *pluginCommand(const struct MIMETYPE *m,
 	cmd = allocMem(len);
 	t = cmd;
 	for (s = m->program; *s; ++s) {
-#if 0
-		if (*s == '*') {
-			strcpy(t, suffix);
-			t += strlen(suffix);
-			continue;
-		}
-#endif
 		if (*s == '%' && s[1] == 'i') {
 			shellProtect(t, infile);
 			t += inlen;
