@@ -49,6 +49,7 @@ document.forms = new Array;
 document.elements = new Array;
 document.anchors = new Array;
 document.divs = new Array;
+document.htmlobjs = new Array;
 document.scripts = new Array;
 document.paragraphs = new Array;
 document.tables = new Array;
@@ -858,6 +859,22 @@ Div.prototype.getAttribute = document.getAttribute;
 Div.prototype.setAttribute = document.setAttribute;
 Div.prototype.cloneNode = document.cloneNode;
 
+HtmlObj.prototype.appendChild = document.appendChild;
+HtmlObj.prototype.apch$ = document.apch$;
+HtmlObj.prototype.insertBefore = document.insertBefore;
+Object.defineProperty(HtmlObj.prototype, "firstChild", {
+get: function() { return this.childNodes[0]; }
+});
+Object.defineProperty(HtmlObj.prototype, "lastChild", {
+get: function() { return this.childNodes[this.childNodes.length-1]; }
+});
+HtmlObj.prototype.hasChildNodes = document.hasChildNodes;
+HtmlObj.prototype.removeChild = document.removeChild;
+HtmlObj.prototype.replaceChild = document.replaceChild;
+HtmlObj.prototype.getAttribute = document.getAttribute;
+HtmlObj.prototype.setAttribute = document.setAttribute;
+HtmlObj.prototype.cloneNode = document.cloneNode;
+
 Script.prototype.getAttribute = document.getAttribute;
 Script.prototype.setAttribute = document.setAttribute;
 Script.prototype.cloneNode = document.cloneNode;
@@ -1033,16 +1050,6 @@ console.info = console.log;
 console.warn = console.log;
 console.error = console.log;
 
-// just a placeholder for now, for code that 
-// expects to be able to create an XHR object
-// Implementation of actual asynchronous HTTP is going to be more work
-
-XMLHttpRequest = function(){
-    this.headers = {};
-    this.responseHeaders = {};
-    this.aborted = false;//non-standard
-};
-
 /* An ok (object keys) function for javascript/dom debugging. */
 /* This is in concert with the jdb command in edbrowse. */
 ok = Object.keys = Object.keys || (function () { 
@@ -1071,3 +1078,151 @@ ok = Object.keys = Object.keys || (function () {
 		}; 
 		})(); 
 
+// ------------ experimental below this point
+// @author Originally implemented by Yehuda Katz
+// And since then, from envjs, by Thatcher et al
+
+var domparser;
+
+XMLHttpRequest = function(){
+    this.headers = {};
+    this.responseHeaders = {};
+    this.aborted = false;//non-standard
+};
+
+// defined by the standard: http://www.w3.org/TR/XMLHttpRequest/#xmlhttprequest
+// but not provided by Firefox.  Safari and others do define it.
+XMLHttpRequest.UNSENT = 0;
+XMLHttpRequest.OPEN = 1;
+XMLHttpRequest.HEADERS_RECEIVED = 2;
+XMLHttpRequest.LOADING = 3;
+XMLHttpRequest.DONE = 4;
+
+XMLHttpRequest.prototype = {
+open: function(method, url, async, user, password){
+this.readyState = 1;
+this.async = false;
+// Note: async is currently hardcoded to false
+// In the implementation in envjs, the line here was:
+// this.async = (async === false)?false:true;
+
+this.method = method || "GET";
+this.url = convert_url(url);
+this.onreadystatechange();
+},
+setRequestHeader: function(header, value){
+this.headers[header] = value;
+},
+send: function(data, parsedoc/*non-standard*/){
+var headerstring = "";
+for (var item in this.headers) {
+var v1=item;
+var v2=this.headers[item];
+headerstring+=v1;
+headerstring+=": ";
+headerstring+=v2;
+headerstring+=",";
+}
+
+var entire_http_response =  document.fetchHTTP(this.url,this.method,headerstring,data);
+
+var http_headers = entire_http_response.split("\r\n\r\n")[0];
+
+var responsebody_array = entire_http_response.split("\r\n\r\n");
+responsebody_array[0] = "";
+var responsebody = responsebody_array.join("\r\n\r\n");
+responsebody = responsebody.trim();
+
+this.responseText = responsebody;
+var hhc = http_headers.split("\r\n");
+var i=0;
+while (i < hhc.length) {
+var value1 = hhc[i]+":";
+var value2 = value1.split(":")[0];
+var value3 = value1.split(":")[1];
+this.responseHeaders[value2] = value3.trim();
+i++;
+}
+
+try{
+this.readyState = 4;
+}catch(e){
+}
+
+
+if ((!this.aborted) && this.responseText.length > 0){
+this.readyState = 4;
+this.onreadystatechange();
+}
+
+},
+abort: function(){
+this.aborted = true;
+},
+onreadystatechange: function(){
+//Instance specific
+},
+getResponseHeader: function(header){
+var rHeader, returnedHeaders;
+if (this.readyState < 3){
+throw new Error("INVALID_STATE_ERR");
+} else {
+returnedHeaders = [];
+for (rHeader in this.responseHeaders) {
+if (rHeader.match(new RegExp(header, "i"))) {
+returnedHeaders.push(this.responseHeaders[rHeader]);
+}
+}
+
+if (returnedHeaders.length){
+return returnedHeaders.join(", ");
+}
+}
+return null;
+},
+getAllResponseHeaders: function(){
+var header, returnedHeaders = [];
+if (this.readyState < 3){
+throw new Error("INVALID_STATE_ERR");
+} else {
+for (header in this.responseHeaders) {
+returnedHeaders.push( header + ": " + this.responseHeaders[header] );
+}
+}
+return returnedHeaders.join("\r\n");
+},
+async: false,
+readyState: 0,
+responseText: "",
+status: 0,
+statusText: ""
+};
+
+convert_url = function(path, base) {
+
+var new_url;
+var sideprotocol =location.protocol;
+var sidehost = location.host;
+var lpl  = location.protocol.length;
+if (lpl ==  0)
+{
+sideprotocol = "http:";
+sideprotocol.length = 5;
+lpl = 5;
+}
+if (sidehost == "")
+{
+sidehost = location.href;
+}
+
+var protocol1= path.substring(0,lpl);
+if (protocol1 == sideprotocol)
+{
+new_url = path;
+// ok - it is fully qualified already
+} else {
+new_url = sideprotocol + '//' + sidehost + path;
+}
+return new_url;
+}
+
